# Copyright (c) 2017 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from kmip.core import attributes
from kmip.core import enums
from kmip.core import utils

from kmip.core.messages.payloads import sign


class TestSignRequestPayload(testtools.TestCase):
    """
    Test suite for the Sign request payload.
    """

    def setUp(self):
        super(TestSignRequestPayload, self).setUp()

        # Encoding obtained in part from KMIP 1.4 testing document,
        # partially cobbled together by hand from other test cases
        # in this code base.
        #
        # This encoding matches the following set of values:
        # Unique Identifier - b4faee10-aa2a-4446-8ad4-0881f3422959
        # Cryptographic Parameters
        #     Cryptographic Algorithm - ECDSA
        # Data - 01020304050607080910111213141516

        self.full_encoding = utils.BytearrayStream(
            b'\x42\x00\x79\x01\x00\x00\x00\x60'
            b'\x42\x00\x94\x07\x00\x00\x00\x24\x62\x34\x66\x61\x65\x65\x31\x30'
            b'\x2D\x61\x61\x32\x61\x2D\x34\x34\x34\x36\x2D\x38\x61\x64\x34\x2D'
            b'\x30\x38\x38\x31\x66\x33\x34\x32\x32\x39\x35\x39\x00\x00\x00\x00'
            b'\x42\x00\x2B\x01\x00\x00\x00\x10'
            b'\x42\x00\x28\x05\x00\x00\x00\x04\x00\x00\x00\x06\x00\x00\x00\x00'
            b'\x42\x00\xC2\x08\x00\x00\x00\x10\x01\x02\x03\x04\x05\x06\x07\x08'
            b'\x09\x10\x11\x12\x13\x14\x15\x16'
        )

        self.minimum_encoding = utils.BytearrayStream(
            b'\x42\x00\x79\x01\x00\x00\x00\x18'
            b'\x42\x00\xC2\x08\x00\x00\x00\x10\x01\x02\x03\x04\x05\x06\x07\x08'
            b'\x09\x10\x11\x12\x13\x14\x15\x16'
        )

        self.empty_encoding = utils.BytearrayStream(
            b'\x42\x00\x79\x01\x00\x00\x00\x00'
        )

    def tearDown(self):
        super(TestSignRequestPayload, self).tearDown()

    def test_init(self):
        """
        Test that a Sign request payload can be constructed with no arguments.
        """
        payload = sign.SignRequestPayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.cryptographic_parameters)
        self.assertEqual(None, payload.data)

    def test_init_with_args(self):
        """
        Test that a Sign request payload can be constructed with valid values.
        """
        payload = sign.SignRequestPayload(
            unique_identifier='00000000-1111-2222-3333-444444444444',
            cryptographic_parameters=attributes.CryptographicParameters(),
            data=b'\x01\x02\x03'
        )

        self.assertEqual(
            '00000000-1111-2222-3333-444444444444',
            payload.unique_identifier
        )
        self.assertEqual(
            attributes.CryptographicParameters(),
            payload.cryptographic_parameters
        )
        self.assertEqual(b'\x01\x02\x03', payload.data)

    def test_invalid_unique_identifier(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the unique identifier of a Sign request payload.
        """
        payload = sign.SignRequestPayload()
        args = (payload, 'unique_identifier', 0)
        self.assertRaisesRegex(
            TypeError,
            "unique identifier must be a string",
            setattr,
            *args
        )

    def test_invalid_cryptographic_parameters(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the cryptographic parameters of a Sign request payload.
        """
        payload = sign.SignRequestPayload()
        args = (payload, 'cryptographic_parameters', b'\x01\x02\x03')
        self.assertRaisesRegex(
            TypeError,
            "cryptographic parameters must be a CryptographicParameters "
            "struct",
            setattr,
            *args
        )

    def test_invalid_data(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the data of a Sign request payload.
        """
        payload = sign.SignRequestPayload()
        args = (payload, 'data', 0)
        self.assertRaisesRegex(
            TypeError,
            "data must be bytes",
            setattr,
            *args
        )

    def test_read(self):
        """
        Test that a Sign request payload can be read from a data stream.
        """
        payload = sign.SignRequestPayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.cryptographic_parameters)
        self.assertEqual(None, payload.data)

        payload.read(self.full_encoding)

        self.assertEqual(
            'b4faee10-aa2a-4446-8ad4-0881f3422959',
            payload.unique_identifier
        )
        self.assertIsNotNone(payload.cryptographic_parameters)
        self.assertEqual(
            enums.CryptographicAlgorithm.ECDSA,
            payload.cryptographic_parameters.cryptographic_algorithm
        )
        self.assertEqual(
            b'\x01\x02\x03\x04\x05\x06\x07\x08'
            b'\x09\x10\x11\x12\x13\x14\x15\x16',
            payload.data
        )

    def test_read_partial(self):
        """
        Test that a Sign request payload can be read from a partial data
        stream containing the minimum required attributes.
        """
        payload = sign.SignRequestPayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.cryptographic_parameters)
        self.assertEqual(None, payload.data)

        payload.read(self.minimum_encoding)

        self.assertEqual(
            b'\x01\x02\x03\x04\x05\x06\x07\x08'
            b'\x09\x10\x11\x12\x13\x14\x15\x16',
            payload.data
        )

    def test_read_invalid(self):
        """
        Test that a ValueError gets raised when a required Sign request
        payload attribute is missing from the payload encoding.
        """
        payload = sign.SignRequestPayload()
        args = (self.empty_encoding, )
        self.assertRaisesRegex(
            ValueError,
            "invalid payload missing the data attribute",
            payload.read,
            *args
        )

    def test_write(self):
        """
        Test that a Sign request payload can be written to a data stream.
        """
        payload = sign.SignRequestPayload(
            unique_identifier='b4faee10-aa2a-4446-8ad4-0881f3422959',
            cryptographic_parameters=attributes.CryptographicParameters(
                cryptographic_algorithm=enums.CryptographicAlgorithm.ECDSA
            ),
            data=b'\x01\x02\x03\x04\x05\x06\x07\x08'
                 b'\x09\x10\x11\x12\x13\x14\x15\x16'
        )
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.full_encoding), len(stream))
        self.assertEqual(str(self.full_encoding), str(stream))

    def test_write_partial(self):
        """
        Test that a partially defined Sign request payload can be written
        to a data stream.
        """
        payload = sign.SignRequestPayload(
            data=b'\x01\x02\x03\x04\x05\x06\x07\x08'
                 b'\x09\x10\x11\x12\x13\x14\x15\x16'
        )
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.minimum_encoding), len(stream))
        self.assertEqual(str(self.minimum_encoding), str(stream))

    def test_write_invalid(self):
        """
        Test that a ValueError gets raised when a required Sign request
        payload attribute is missing when encoding the payload.
        """
        payload = sign.SignRequestPayload()
        stream = utils.BytearrayStream()
        args = (stream, )
        self.assertRaisesRegex(
            ValueError,
            "invalid payload missing the data attribute",
            payload.write,
            *args
        )

    def test_equal_on_equal(self):
        """
        Test that the equality operator returns True when comparing two
        Sign request payloads with the same data.
        """
        a = sign.SignRequestPayload()
        b = sign.SignRequestPayload()

        self.assertTrue(a == b)
        self.assertTrue(b == a)

        a = sign.SignRequestPayload(
            unique_identifier='b4faee10-aa2a-4446-8ad4-0881f3422959',
            cryptographic_parameters=attributes.CryptographicParameters(
                cryptographic_algorithm=enums.CryptographicAlgorithm.ECDSA
            ),
            data=b'\x01\x02\x03\x04\x05\x06\x07\x08'
                 b'\x09\x10\x11\x12\x13\x14\x15\x16'
        )
        b = sign.SignRequestPayload(
            unique_identifier='b4faee10-aa2a-4446-8ad4-0881f3422959',
            cryptographic_parameters=attributes.CryptographicParameters(
                cryptographic_algorithm=enums.CryptographicAlgorithm.ECDSA
            ),
            data=b'\x01\x02\x03\x04\x05\x06\x07\x08'
                 b'\x09\x10\x11\x12\x13\x14\x15\x16'
        )

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_not_equal_unique_identifier(self):
        """
        Test that the equality operator returns False when comparing two
        Sign request payloads with different unique identifiers.
        """
        a = sign.SignRequestPayload(
            unique_identifier='a'
        )
        b = sign.SignRequestPayload(
            unique_identifier='b'
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_cryptographic_parameters(self):
        """
        Test that the equality operator returns False when comparing two
        Sign request payloads with cryptographic parameters.
        """
        a = sign.SignRequestPayload(
            cryptographic_parameters=attributes.CryptographicParameters(
                hashing_algorithm=enums.HashingAlgorithm.MD5
            )
        )
        b = sign.SignRequestPayload(
            cryptographic_parameters=attributes.CryptographicParameters(
                cryptographic_algorithm=enums.CryptographicAlgorithm.ECDSA
            )
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_data(self):
        """
        Test that the equality operator returns False when comparing two
        Sign request payloads with different data.
        """
        a = sign.SignRequestPayload(data=b'\x01')
        b = sign.SignRequestPayload(data=b'\xFF')

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_type_mismatch(self):
        """
        Test that the equality operator returns False when comparing two
        Sign request payloads with different types.
        """
        a = sign.SignRequestPayload()
        b = 'invalid'

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_not_equal_on_equal(self):
        """
        Test that the inequality operator returns False when comparing two
        Sign request payloads with the same data.
        """
        a = sign.SignRequestPayload()
        b = sign.SignRequestPayload()

        self.assertFalse(a != b)
        self.assertFalse(b != a)

        a = sign.SignRequestPayload(
            unique_identifier='b4faee10-aa2a-4446-8ad4-0881f3422959',
            cryptographic_parameters=attributes.CryptographicParameters(
                cryptographic_algorithm=enums.CryptographicAlgorithm.ECDSA
            ),
            data=b'\x01\x02\x03\x04\x05\x06\x07\x08'
                 b'\x09\x10\x11\x12\x13\x14\x15\x16'
        )
        b = sign.SignRequestPayload(
            unique_identifier='b4faee10-aa2a-4446-8ad4-0881f3422959',
            cryptographic_parameters=attributes.CryptographicParameters(
                cryptographic_algorithm=enums.CryptographicAlgorithm.ECDSA
            ),
            data=b'\x01\x02\x03\x04\x05\x06\x07\x08'
                 b'\x09\x10\x11\x12\x13\x14\x15\x16'
        )

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_not_equal_unique_identifier(self):
        """
        Test that the inequality operator returns True when comparing two
        Sign request payloads with different unique identifiers.
        """
        a = sign.SignRequestPayload(
            unique_identifier='a'
        )
        b = sign.SignRequestPayload(
            unique_identifier='b'
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_cryptographic_parameters(self):
        """
        Test that the equality operator returns False when comparing two
        Sign request payloads with cryptographic parameters.
        """
        a = sign.SignRequestPayload(
            cryptographic_parameters=attributes.CryptographicParameters(
                hashing_algorithm=enums.HashingAlgorithm.MD5
            )
        )
        b = sign.SignRequestPayload(
            cryptographic_parameters=attributes.CryptographicParameters(
                cryptographic_algorithm=enums.CryptographicAlgorithm.ECDSA
            )
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_data(self):
        """
        Test that the inequality operator returns True when comparing two
        Sign request payloads with different data.
        """
        a = sign.SignRequestPayload(data=b'\x01')
        b = sign.SignRequestPayload(data=b'\xFF')

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_type_mismatch(self):
        """
        Test that the inequality operator returns True when comparing two
        Sign request payloads with different types.
        """
        a = sign.SignRequestPayload()
        b = 'invalid'

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_repr(self):
        """
        Test that repr can be applied to a Sign request payload.
        """
        payload = sign.SignRequestPayload(
            unique_identifier='b4faee10-aa2a-4446-8ad4-0881f3422959',
            cryptographic_parameters=attributes.CryptographicParameters(
                cryptographic_algorithm=enums.CryptographicAlgorithm.ECDSA
            ),
            data=b'\x01\x02\x03\x04\x05\x06\x07\x08'
        )
        expected = (
            "SignRequestPayload("
            "unique_identifier='b4faee10-aa2a-4446-8ad4-0881f3422959', "
            "cryptographic_parameters=CryptographicParameters("
            "block_cipher_mode=None, padding_method=None, "
            "hashing_algorithm=None, key_role_type=None, "
            "digital_signature_algorithm=None, "
            "cryptographic_algorithm=CryptographicAlgorithm.ECDSA, "
            "random_iv=None, iv_length=None, tag_length=None, "
            "fixed_field_length=None, invocation_field_length=None, "
            "counter_length=None, initial_counter_value=None), "
            "data=" + str(b'\x01\x02\x03\x04\x05\x06\x07\x08') + ")"
        )
        observed = repr(payload)

        self.assertEqual(expected, observed)

    def test_str(self):
        """
        Test that str can be applied to a Sign request payload.
        """
        crypto_params = attributes.CryptographicParameters(
             cryptographic_algorithm=enums.CryptographicAlgorithm.ECDSA
        )
        payload = sign.SignRequestPayload(
            unique_identifier='b4faee10-aa2a-4446-8ad4-0881f3422959',
            cryptographic_parameters=crypto_params,
            data=b'\x01\x02\x03\x04\x05\x06\x07\x08',
        )

        expected = str({
            'unique_identifier': 'b4faee10-aa2a-4446-8ad4-0881f3422959',
            'cryptographic_parameters': crypto_params,
            'data': b'\x01\x02\x03\x04\x05\x06\x07\x08'
        })
        observed = str(payload)

        self.assertEqual(expected, observed)


class TestSignResponsePayload(testtools.TestCase):
    """
    Test suite for the Sign response payload.
    """

    def setUp(self):
        super(TestSignResponsePayload, self).setUp()
        # Encoding obtained in part from KMIP 1.4 testing document,
        # partially cobbled together by hand from other test cases
        # in this code base.
        #
        # This encoding matches the following set of values:
        # Unique Identifier - b4faee10-aa2a-4446-8ad4-0881f3422959
        # Signature Data - 01020304050607080910111213141516

        self.full_encoding = utils.BytearrayStream(
            b'\x42\x00\x7C\x01\x00\x00\x00\x48'
            b'\x42\x00\x94\x07\x00\x00\x00\x24\x62\x34\x66\x61\x65\x65\x31\x30'
            b'\x2D\x61\x61\x32\x61\x2D\x34\x34\x34\x36\x2D\x38\x61\x64\x34\x2D'
            b'\x30\x38\x38\x31\x66\x33\x34\x32\x32\x39\x35\x39\x00\x00\x00\x00'
            b'\x42\x00\xC3\x08\x00\x00\x00\x10\x01\x02\x03\x04\x05\x06\x07\x08'
            b'\x09\x10\x11\x12\x13\x14\x15\x16'
        )
        self.incomplete_encoding = utils.BytearrayStream(
            b'\x42\x00\x7C\x01\x00\x00\x00\x30'
            b'\x42\x00\x94\x07\x00\x00\x00\x24\x62\x34\x66\x61\x65\x65\x31\x30'
            b'\x2D\x61\x61\x32\x61\x2D\x34\x34\x34\x36\x2D\x38\x61\x64\x34\x2D'
            b'\x30\x38\x38\x31\x66\x33\x34\x32\x32\x39\x35\x39\x00\x00\x00\x00'
        )
        self.empty_encoding = utils.BytearrayStream(
            b'\x42\x00\x7C\x01\x00\x00\x00\x00'
        )

    def tearDown(self):
        super(TestSignResponsePayload, self).tearDown()

    def test_init(self):
        """
        Test that a Sign response payload can be constructed with no
        arguments.
        """
        payload = sign.SignResponsePayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.signature_data)

    def test_init_with_args(self):
        """
        Test that a Sign response payload can be constructed with valid
        values.
        """
        payload = sign.SignResponsePayload(
            unique_identifier='00000000-1111-2222-3333-444444444444',
            signature_data=b'\x01\x02\x03'
        )

        self.assertEqual(
            '00000000-1111-2222-3333-444444444444',
            payload.unique_identifier
        )
        self.assertEqual(b'\x01\x02\x03', payload.signature_data)

    def test_invalid_unique_identifier(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the unique identifier of a Sign response payload.
        """
        payload = sign.SignResponsePayload()
        args = (payload, 'unique_identifier', 0)
        self.assertRaisesRegex(
            TypeError,
            "unique identifier must be a string",
            setattr,
            *args
        )

    def test_invalid_signature_data(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the signature data of a Sign response payload.
        """
        payload = sign.SignResponsePayload()
        args = (payload, 'signature_data', 0)
        self.assertRaisesRegex(
            TypeError,
            "signature data must be bytes",
            setattr,
            *args
        )

    def test_read(self):
        """
        Test that a Sign response payload can be read from a data stream.
        """
        payload = sign.SignResponsePayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.signature_data)

        payload.read(self.full_encoding)

        self.assertEqual(
            'b4faee10-aa2a-4446-8ad4-0881f3422959',
            payload.unique_identifier
        )
        self.assertEqual(
            b'\x01\x02\x03\x04\x05\x06\x07\x08'
            b'\x09\x10\x11\x12\x13\x14\x15\x16',
            payload.signature_data
        )

    def test_read_invalid(self):
        """
        Test that a ValueError gets raised when required Sign response
        payload attributes are missing from the payload encoding.
        """
        payload = sign.SignResponsePayload()
        args = (self.empty_encoding, )
        self.assertRaisesRegex(
            ValueError,
            "invalid payload missing the unique identifier attribute",
            payload.read,
            *args
        )

        payload = sign.SignResponsePayload()
        args = (self.incomplete_encoding, )
        self.assertRaisesRegex(
            ValueError,
            "invalid payload missing the signature data attribute",
            payload.read,
            *args
        )

    def test_write(self):
        """
        Test that a Sign response payload can be written to a data stream.
        """
        payload = sign.SignResponsePayload(
            unique_identifier='b4faee10-aa2a-4446-8ad4-0881f3422959',
            signature_data=b'\x01\x02\x03\x04\x05\x06\x07\x08'
                           b'\x09\x10\x11\x12\x13\x14\x15\x16'
        )
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.full_encoding), len(stream))
        self.assertEqual(str(self.full_encoding), str(stream))

    def test_write_invalid(self):
        """
        Test that a ValueError gets raised when a required Sign response
        payload attribute is missing when encoding the payload.
        """
        payload = sign.SignResponsePayload()
        stream = utils.BytearrayStream()
        args = (stream, )
        self.assertRaisesRegex(
            ValueError,
            "invalid payload missing the unique identifier attribute",
            payload.write,
            *args
        )

    def test_equal_on_equal(self):
        """
        Test that the equality operator returns True when comparing two
        equal sign response payloads
        """
        encoding = utils.BytearrayStream(self.full_encoding.buffer)
        a = sign.SignResponsePayload()
        b = sign.SignResponsePayload()

        self.assertTrue(a == b)
        self.assertTrue(b == a)

        a.read(encoding)
        b.read(self.full_encoding)

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_not_equal_unique_identifier(self):
        """
        Test that the equality operator returns False when comparing two
        sign response payloads with different unique_identifier.
        """
        a = sign.SignResponsePayload(unique_identifier='a',
                                     signature_data=b'\x01')

        b = sign.SignResponsePayload(unique_identifier='b',
                                     signature_data=b'\x01')

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_signature_data(self):
        """
        Test that the equality operator returns False when comparing two
        sign response payloads with different signature_data.
        """
        a = sign.SignResponsePayload(unique_identifier='a',
                                     signature_data=b'\x01')
        b = sign.SignResponsePayload(unique_identifier='a',
                                     signature_data=b'\x02')

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_type_mismatch(self):
        """
        Test that the equality operator returns False when comparing a sign
        response payload to another type.
        """
        a = sign.SignResponsePayload()
        b = 'invalid'

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_not_equal_on_equal(self):
        """
        Test that the inequality operator returns false when comparing two
        equal sign response payloads.
        """
        encoding = utils.BytearrayStream(self.full_encoding.buffer)
        a = sign.SignResponsePayload()
        b = sign.SignResponsePayload()

        self.assertFalse(a != b)
        self.assertFalse(b != a)

        a.read(encoding)
        b.read(self.full_encoding)

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_not_equal_unique_identifier(self):
        """
        Test that the inequality operator returns True when comparing two
        sign response payloads with different unique_identifier.
        """
        a = sign.SignResponsePayload(unique_identifier='a',
                                     signature_data=b'\x01')

        b = sign.SignResponsePayload(unique_identifier='b',
                                     signature_data=b'\x01')

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_signature_data(self):
        """
        Test that the inequality operator returns True when comparing two
        sign response payloads with different signature_data.
        """
        a = sign.SignResponsePayload(unique_identifier='a',
                                     signature_data=b'\x01')
        b = sign.SignResponsePayload(unique_identifier='a',
                                     signature_data=b'\x02')

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_type_mismatch(self):
        """
        Test that the inequality operator returns True when comparing a
        sign response payload to a different type.
        """
        a = sign.SignResponsePayload()
        b = 'invalid'

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_repr(self):
        """
        Test that repr can applied to a sign response payload.
        """
        payload = sign.SignResponsePayload(
            unique_identifier='00000000-1111-2222-3333-444444444444',
            signature_data=b'\x01\x02\x03'
        )

        expected = (
            "SignResponsePayload("
            "unique_identifier='00000000-1111-2222-3333-444444444444', "
            "signature_data="+str(b'\x01\x02\x03') + ")"
        )

        observed = repr(payload)
        self.assertEqual(expected, observed)

    def test_str(self):
        """
        Test that str can be applied to a sign response payload.
        """
        payload = sign.SignResponsePayload(
            unique_identifier='00000000-1111-2222-3333-444444444444',
            signature_data=b'\x01\x02\x03'
        )

        expected = str({
            'unique_identifier': '00000000-1111-2222-3333-444444444444',
            'signature_data': b'\x01\x02\x03'
        })

        observed = str(payload)
        self.assertEqual(expected, observed)



# Copyright (c) 2016 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from kmip.core import enums
from kmip.pie.sqltypes import ManagedObjectName


class TestSqlTypesManagedObjectName(testtools.TestCase):
    """
    Test suite for objects in sqltypes.py.
    """

    def test_equal_on_equal(self):
        """
        Test that the equality operator returns True when comparing two
        ManagedObjectName objects with the same data.
        """
        a = ManagedObjectName('a', 0)
        b = ManagedObjectName('a', 0)
        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_not_equal_name(self):
        """
        Test that the equality operator returns False when comparing two
        ManagedObjectName objects with different names.
        """
        a = ManagedObjectName('a', 0)
        b = ManagedObjectName('b', 0)
        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_index(self):
        """
        Test that the equality operator returns False when comparing two
        ManagedObjectName objects with different indices.
        """
        a = ManagedObjectName('a', 0)
        b = ManagedObjectName('a', 1)
        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_name_type(self):
        """
        Test that the equality operator returns False when comparing two
        ManagedObjectName objects with different name types.
        """
        a = ManagedObjectName('a', 0, enums.NameType.UNINTERPRETED_TEXT_STRING)
        b = ManagedObjectName('a', 0, enums.NameType.URI)
        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_class_type(self):
        """
        Test that the equality operator returns False when comparing a
        ManagedObjectName object with a different type of object.
        """
        a = ManagedObjectName('a', 0, enums.NameType.UNINTERPRETED_TEXT_STRING)
        b = 'foo'
        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_not_equal_on_equal(self):
        """
        Test that the not equal operator returns False when comparing two
        ManagedObjectName objects with the same data.
        """
        a = ManagedObjectName('a', 0)
        b = ManagedObjectName('a', 0)
        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_not_equal_name(self):
        """
        Test that the not equal operator returns True when comparing two
        ManagedObjectName objects with different names.
        """
        a = ManagedObjectName('a', 0)
        b = ManagedObjectName('b', 0)
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_index(self):
        """
        Test that the not equal operator returns True when comparing two
        ManagedObjectName objects with different indices.
        """
        a = ManagedObjectName('a', 0)
        b = ManagedObjectName('a', 1)
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_name_type(self):
        """
        Test that the not equal operator returns True when comparing two
        ManagedObjectName objects with different name types.
        """
        a = ManagedObjectName('a', 0, enums.NameType.UNINTERPRETED_TEXT_STRING)
        b = ManagedObjectName('a', 0, enums.NameType.URI)
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_class_type(self):
        """
        Test that the not equal operator returns False when comparing a
        ManagedObjectName object with a different type of object.
        """
        a = ManagedObjectName('a', 0, enums.NameType.UNINTERPRETED_TEXT_STRING)
        b = 'foo'
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_repr(self):
        """
        Test that __repr__ is implemented.
        """
        a = ManagedObjectName('a', 0, enums.NameType.UNINTERPRETED_TEXT_STRING)
        repr(a)



# Copyright (c) 2019 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sqlalchemy
import testtools

from kmip.pie import objects
from kmip.pie import sqltypes


class TestApplicationSpecificInformation(testtools.TestCase):
    """
    Test suite for ApplicationSpecificInformation.
    """

    def setUp(self):
        super(TestApplicationSpecificInformation, self).setUp()

    def tearDown(self):
        super(TestApplicationSpecificInformation, self).tearDown()

    def test_init(self):
        """
        Test that an ApplicationSpecificInformation object can be instantiated.
        """
        app_specific_info = objects.ApplicationSpecificInformation()

        self.assertIsNone(app_specific_info.application_namespace)
        self.assertIsNone(app_specific_info.application_data)

    def test_invalid_application_namespace(self):
        """
        Test that a TypeError is raised when an invalid application namespace
        value is used to construct an ApplicationSpecificInformation attribute.
        """
        kwargs = {"application_namespace": []}
        self.assertRaisesRegex(
            TypeError,
            "The application namespace must be a string.",
            objects.ApplicationSpecificInformation,
            **kwargs
        )

        args = (
            objects.ApplicationSpecificInformation(),
            "application_namespace",
            []
        )
        self.assertRaisesRegex(
            TypeError,
            "The application namespace must be a string.",
            setattr,
            *args
        )

    def test_invalid_application_data(self):
        """
        Test that a TypeError is raised when an invalid application data value
        is used to construct an ApplicationSpecificInformation attribute.
        """
        kwargs = {"application_data": []}
        self.assertRaisesRegex(
            TypeError,
            "The application data must be a string.",
            objects.ApplicationSpecificInformation,
            **kwargs
        )

        args = (
            objects.ApplicationSpecificInformation(),
            "application_data",
            []
        )
        self.assertRaisesRegex(
            TypeError,
            "The application data must be a string.",
            setattr,
            *args
        )

    def test_repr(self):
        """
        Test that repr can be applied to an ApplicationSpecificInformation
        attribute.
        """
        app_specific_info = objects.ApplicationSpecificInformation(
            application_namespace="ssl",
            application_data="www.example.com"
        )

        args = [
            "application_namespace='{}'".format("ssl"),
            "application_data='{}'".format("www.example.com")
        ]

        expected = "ApplicationSpecificInformation({})".format(", ".join(args))
        observed = repr(app_specific_info)

        self.assertEqual(expected, observed)

    def test_str(self):
        """
        Test that str can be applied to an ApplicationSpecificInformation
        attribute.
        """
        app_specific_info = objects.ApplicationSpecificInformation(
            application_namespace="ssl",
            application_data="www.example.com"
        )

        expected = str(
            {
                "application_namespace": "ssl",
                "application_data": "www.example.com"
            }
        )
        observed = str(app_specific_info)

        self.assertEqual(expected, observed)

    def test_comparison_on_equal(self):
        """
        Test that the equality/inequality operators return True/False when
        comparing two ApplicationSpecificInformation attributes with the same
        data.
        """
        a = objects.ApplicationSpecificInformation()
        b = objects.ApplicationSpecificInformation()

        self.assertTrue(a == b)
        self.assertTrue(b == a)
        self.assertFalse(a != b)
        self.assertFalse(b != a)

        a = objects.ApplicationSpecificInformation(
            application_namespace="ssl",
            application_data="www.example.com"
        )
        b = objects.ApplicationSpecificInformation(
            application_namespace="ssl",
            application_data="www.example.com"
        )

        self.assertTrue(a == b)
        self.assertTrue(b == a)
        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_comparison_on_different_application_namespaces(self):
        """
        Test that the equality/inequality operators return False/True when
        comparing two ApplicationSpecificInformation attributes with different
        application namespaces.
        """
        a = objects.ApplicationSpecificInformation(
            application_namespace="a"
        )
        b = objects.ApplicationSpecificInformation(
            application_namespace="b"
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_comparison_on_different_application_data(self):
        """
        Test that the equality/inequality operators return False/True when
        comparing two ApplicationSpecificInformation attributes with different
        application data.
        """
        a = objects.ApplicationSpecificInformation(
            application_data="a"
        )
        b = objects.ApplicationSpecificInformation(
            application_data="b"
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_comparison_on_type_mismatch(self):
        """
        Test that the equality/inequality operators return False/True when
        comparing an ApplicationSpecificInformation attribute to a non
        ApplicationSpecificInformation attribute.
        """
        a = objects.ApplicationSpecificInformation()
        b = "invalid"

        self.assertFalse(a == b)
        self.assertFalse(b == a)
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_save(self):
        """
        Test that an ApplicationSpecificInformation attribute can be saved
        using SQLAlchemy. This test will add an attribute instance to the
        database, verify that no exceptions are thrown, and check that its
        ID was set.
        """
        app_specific_info = objects.ApplicationSpecificInformation(
            application_namespace="ssl",
            application_data="www.example.com"
        )

        engine = sqlalchemy.create_engine("sqlite:///:memory:", echo=True)
        sqltypes.Base.metadata.create_all(engine)

        session = sqlalchemy.orm.sessionmaker(bind=engine)()
        session.add(app_specific_info)
        session.commit()

        self.assertIsNotNone(app_specific_info.id)

    def test_get(self):
        """
        Test that an ApplicationSpecificInformation attribute can be saved
        and then retrieved using SQLAlchemy. This test adds the attribute to
        the database and then retrieves it by ID and verifies its values.
        """
        app_specific_info = objects.ApplicationSpecificInformation(
            application_namespace="ssl",
            application_data="www.example.com"
        )

        engine = sqlalchemy.create_engine("sqlite:///:memory:", echo=True)
        sqltypes.Base.metadata.create_all(engine)

        session = sqlalchemy.orm.sessionmaker(bind=engine)()
        session.add(app_specific_info)
        session.commit()

        # Grab the ID now before making a new session to avoid a Detached error
        # See http://sqlalche.me/e/bhk3 for more info.
        app_specific_info_id = app_specific_info.id

        session = sqlalchemy.orm.sessionmaker(bind=engine)()
        retrieved_info = session.query(
            objects.ApplicationSpecificInformation
        ).filter(
            objects.ApplicationSpecificInformation.id == app_specific_info_id
        ).one()
        session.commit()

        self.assertEqual("ssl", retrieved_info.application_namespace)
        self.assertEqual("www.example.com", retrieved_info.application_data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from kmip.core import enums
from kmip.core import objects
from kmip.core import primitives
from kmip.core import utils

from kmip.core.messages import payloads


class TestRekeyRequestPayload(testtools.TestCase):
    """
    Test suite for the Rekey request payload.
    """

    def setUp(self):
        super(TestRekeyRequestPayload, self).setUp()

        # Encoding obtained from the KMIP 1.1 testing document,
        # Sections 9.2 and 9.4.
        #
        # This encoding matches the following set of values:
        # Request Payload
        #     Unique Identifier - 1346d253-69d6-474c-8cd5-ad475a3e0a81
        #     Offset - 0
        #     Template Attribute
        #         Attribute
        #             Attribute Name - Activation Date
        #             Attribute Value - Sun Jan 01 12:00:00 CET 2006
        #         Attribute
        #             Attribute Name - Process Start Date
        #             Attribute Value - Sun Jan 01 12:00:00 CET 2006
        #         Attribute
        #             Attribute Name - Protect Stop Date
        #             Attribute Value - Wed Jan 01 12:00:00 CET 2020
        #         Attribute
        #             Attribute Name - Deactivation Date
        #             Attribute Value - Wed Jan 01 12:00:00 CET 2020

        self.full_encoding = utils.BytearrayStream(
            b'\x42\x00\x79\x01\x00\x00\x01\x20'
            b'\x42\x00\x94\x07\x00\x00\x00\x24'
            b'\x31\x33\x34\x36\x64\x32\x35\x33\x2D\x36\x39\x64\x36\x2D\x34\x37'
            b'\x34\x63\x2D\x38\x63\x64\x35\x2D\x61\x64\x34\x37\x35\x61\x33\x65'
            b'\x30\x61\x38\x31\x00\x00\x00\x00'
            b'\x42\x00\x58\x0A\x00\x00\x00\x04\x00\x00\x00\x00\x00\x00\x00\x00'
            b'\x42\x00\x91\x01\x00\x00\x00\xD8'
            b'\x42\x00\x08\x01\x00\x00\x00\x28'
            b'\x42\x00\x0A\x07\x00\x00\x00\x0F'
            b'\x41\x63\x74\x69\x76\x61\x74\x69\x6F\x6E\x20\x44\x61\x74\x65\x00'
            b'\x42\x00\x0B\x09\x00\x00\x00\x08\x00\x00\x00\x00\x43\xB7\xB6\x30'
            b'\x42\x00\x08\x01\x00\x00\x00\x30'
            b'\x42\x00\x0A\x07\x00\x00\x00\x12'
            b'\x50\x72\x6F\x63\x65\x73\x73\x20\x53\x74\x61\x72\x74\x20\x44\x61'
            b'\x74\x65\x00\x00\x00\x00\x00\x00'
            b'\x42\x00\x0B\x09\x00\x00\x00\x08\x00\x00\x00\x00\x43\xB7\xB6\x30'
            b'\x42\x00\x08\x01\x00\x00\x00\x30'
            b'\x42\x00\x0A\x07\x00\x00\x00\x11'
            b'\x50\x72\x6F\x74\x65\x63\x74\x20\x53\x74\x6F\x70\x20\x44\x61\x74'
            b'\x65\x00\x00\x00\x00\x00\x00\x00'
            b'\x42\x00\x0B\x09\x00\x00\x00\x08\x00\x00\x00\x00\x5E\x0C\x7B\xB0'
            b'\x42\x00\x08\x01\x00\x00\x00\x30'
            b'\x42\x00\x0A\x07\x00\x00\x00\x11'
            b'\x44\x65\x61\x63\x74\x69\x76\x61\x74\x69\x6F\x6E\x20\x44\x61\x74'
            b'\x65\x00\x00\x00\x00\x00\x00\x00'
            b'\x42\x00\x0B\x09\x00\x00\x00\x08\x00\x00\x00\x00\x5E\x0C\x7B\xB0'
        )

        # Encoding obtained from the KMIP 1.1 testing document, Section 9.1.
        #
        # This encoding matches the following set of values:
        # Request Payload
        #     Unique Identifier - 964d3dd2-5f06-4529-8bb8-ae630b6ca2e0

        self.partial_encoding = utils.BytearrayStream(
            b'\x42\x00\x79\x01\x00\x00\x00\x30'
            b'\x42\x00\x94\x07\x00\x00\x00\x24'
            b'\x39\x36\x34\x64\x33\x64\x64\x32\x2D\x35\x66\x30\x36\x2D\x34\x35'
            b'\x32\x39\x2D\x38\x62\x62\x38\x2D\x61\x65\x36\x33\x30\x62\x36\x63'
            b'\x61\x32\x65\x30\x00\x00\x00\x00'
        )

        self.empty_encoding = utils.BytearrayStream(
            b'\x42\x00\x79\x01\x00\x00\x00\x00'
        )

    def tearDown(self):
        super(TestRekeyRequestPayload, self).tearDown()

    def test_init(self):
        """
        Test that a Rekey request payload can be constructed with no arguments.
        """
        payload = payloads.RekeyRequestPayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.offset)
        self.assertEqual(None, payload.template_attribute)

    def test_init_with_args(self):
        """
        Test that a Rekey request payload can be constructed with valid values.
        """
        payload = payloads.RekeyRequestPayload(
            unique_identifier='00000000-2222-4444-6666-888888888888',
            offset=0,
            template_attribute=objects.TemplateAttribute()
        )

        self.assertEqual(
            '00000000-2222-4444-6666-888888888888',
            payload.unique_identifier
        )
        self.assertEqual(0, payload.offset)
        self.assertEqual(
            objects.TemplateAttribute(),
            payload.template_attribute
        )

    def test_invalid_unique_identifier(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the unique identifier of a Rekey request payload.
        """
        kwargs = {'unique_identifier': 0}
        self.assertRaisesRegex(
            TypeError,
            "Unique identifier must be a string.",
            payloads.RekeyRequestPayload,
            **kwargs
        )

        args = (payloads.RekeyRequestPayload(), 'unique_identifier', 0)
        self.assertRaisesRegex(
            TypeError,
            "Unique identifier must be a string.",
            setattr,
            *args
        )

    def test_invalid_offset(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the offset of a Rekey request payload.
        """
        kwargs = {'offset': 'invalid'}
        self.assertRaisesRegex(
            TypeError,
            "Offset must be an integer.",
            payloads.RekeyRequestPayload,
            **kwargs
        )

        args = (payloads.RekeyRequestPayload(), 'offset', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "Offset must be an integer.",
            setattr,
            *args
        )

    def test_invalid_template_attribute(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the template attribute of a Rekey request payload.
        """
        kwargs = {'template_attribute': 'invalid'}
        self.assertRaisesRegex(
            TypeError,
            "Template attribute must be a TemplateAttribute struct.",
            payloads.RekeyRequestPayload,
            **kwargs
        )

        args = (
            payloads.RekeyRequestPayload(),
            'template_attribute',
            'invalid'
        )
        self.assertRaisesRegex(
            TypeError,
            "Template attribute must be a TemplateAttribute struct.",
            setattr,
            *args
        )

    def test_read(self):
        """
        Test that a Rekey request payload can be read from a data stream.
        """
        payload = payloads.RekeyRequestPayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.offset)
        self.assertEqual(None, payload.template_attribute)

        payload.read(self.full_encoding)

        self.assertEqual(
            '1346d253-69d6-474c-8cd5-ad475a3e0a81',
            payload.unique_identifier
        )
        self.assertEqual(0, payload.offset)
        self.assertEqual(
            objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Activation Date'
                        ),
                        attribute_value=primitives.DateTime(
                            value=1136113200,
                            tag=enums.Tags.ACTIVATION_DATE
                        )
                    ),
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Process Start Date'
                        ),
                        attribute_value=primitives.DateTime(
                            value=1136113200,
                            tag=enums.Tags.PROCESS_START_DATE
                        )
                    ),
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Protect Stop Date'
                        ),
                        attribute_value=primitives.DateTime(
                            value=1577876400,
                            tag=enums.Tags.PROTECT_STOP_DATE
                        )
                    ),
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Deactivation Date'
                        ),
                        attribute_value=primitives.DateTime(
                            value=1577876400,
                            tag=enums.Tags.DEACTIVATION_DATE
                        )
                    )
                ]
            ),
            payload.template_attribute
        )

    def test_read_partial(self):
        """
        Test that a Rekey request payload can be read from a partial data
        stream.
        """
        payload = payloads.RekeyRequestPayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.offset)
        self.assertEqual(None, payload.template_attribute)

        payload.read(self.partial_encoding)

        self.assertEqual(
            '964d3dd2-5f06-4529-8bb8-ae630b6ca2e0',
            payload.unique_identifier
        )
        self.assertEqual(None, payload.offset)
        self.assertEqual(None, payload.template_attribute)

    def test_read_empty(self):
        """
        Test that a Rekey request payload can be read from an empty data
        stream.
        """
        payload = payloads.RekeyRequestPayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.offset)
        self.assertEqual(None, payload.template_attribute)

        payload.read(self.empty_encoding)

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.offset)
        self.assertEqual(None, payload.template_attribute)

    def test_write(self):
        """
        Test that a Rekey request payload can be written to a data stream.
        """
        payload = payloads.RekeyRequestPayload(
            unique_identifier='1346d253-69d6-474c-8cd5-ad475a3e0a81',
            offset=0,
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Activation Date'
                        ),
                        attribute_value=primitives.DateTime(
                            value=1136113200,
                            tag=enums.Tags.ACTIVATION_DATE
                        )
                    ),
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Process Start Date'
                        ),
                        attribute_value=primitives.DateTime(
                            value=1136113200,
                            tag=enums.Tags.PROCESS_START_DATE
                        )
                    ),
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Protect Stop Date'
                        ),
                        attribute_value=primitives.DateTime(
                            value=1577876400,
                            tag=enums.Tags.PROTECT_STOP_DATE
                        )
                    ),
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Deactivation Date'
                        ),
                        attribute_value=primitives.DateTime(
                            value=1577876400,
                            tag=enums.Tags.DEACTIVATION_DATE
                        )
                    )
                ]
            )
        )
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.full_encoding), len(stream))
        self.assertEqual(str(self.full_encoding), str(stream))

    def test_write_partial(self):
        """
        Test that a partial Rekey request payload can be written to a data
        stream.
        """
        payload = payloads.RekeyRequestPayload(
            unique_identifier='964d3dd2-5f06-4529-8bb8-ae630b6ca2e0'
        )
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.partial_encoding), len(stream))
        self.assertEqual(str(self.partial_encoding), str(stream))

    def test_write_empty(self):
        """
        Test that an empty Rekey request payload can be written to a data
        stream.
        """
        payload = payloads.RekeyRequestPayload()
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.empty_encoding), len(stream))
        self.assertEqual(str(self.empty_encoding), str(stream))

    def test_equal_on_equal(self):
        """
        Test that the equality operator returns True when comparing two Rekey
        request payloads with the same data.
        """
        a = payloads.RekeyRequestPayload()
        b = payloads.RekeyRequestPayload()

        self.assertTrue(a == b)
        self.assertTrue(b == a)

        a = payloads.RekeyRequestPayload(
            unique_identifier='1346d253-69d6-474c-8cd5-ad475a3e0a81',
            offset=0,
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Activation Date'
                        ),
                        attribute_value=primitives.DateTime(
                            value=1136113200,
                            tag=enums.Tags.ACTIVATION_DATE
                        )
                    )
                ]
            )
        )
        b = payloads.RekeyRequestPayload(
            unique_identifier='1346d253-69d6-474c-8cd5-ad475a3e0a81',
            offset=0,
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Activation Date'
                        ),
                        attribute_value=primitives.DateTime(
                            value=1136113200,
                            tag=enums.Tags.ACTIVATION_DATE
                        )
                    )
                ]
            )
        )

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_not_equal_unique_identifier(self):
        """
        Test that the equality operator returns False when comparing two Rekey
        request payloads with different unique identifiers.
        """
        a = payloads.RekeyRequestPayload(
            unique_identifier='a'
        )
        b = payloads.RekeyRequestPayload(
            unique_identifier='b'
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_offset(self):
        """
        Test that the equality operator returns False when comparing two Rekey
        request payloads with different offsets.
        """
        a = payloads.RekeyRequestPayload(
            offset=0
        )
        b = payloads.RekeyRequestPayload(
            offset=1
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_template_attribute(self):
        """
        Test that the equality operator returns False when comparing two Rekey
        request payloads with different template attributes.
        """
        a = payloads.RekeyRequestPayload(
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Protect Stop Date'
                        ),
                        attribute_value=primitives.DateTime(
                            value=1577876400,
                            tag=enums.Tags.PROTECT_STOP_DATE
                        )
                    )
                ]
            )
        )
        b = payloads.RekeyRequestPayload(
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Deactivation Date'
                        ),
                        attribute_value=primitives.DateTime(
                            value=1577876400,
                            tag=enums.Tags.DEACTIVATION_DATE
                        )
                    )
                ]
            )
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_type_mismatch(self):
        """
        Test that the equality operator returns False when comparing two Rekey
        request payloads with different types.
        """
        a = payloads.RekeyRequestPayload()
        b = 'invalid'

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_not_equal_on_equal(self):
        """
        Test that the inequality operator returns False when comparing two
        Rekey request payloads with the same data.
        """
        a = payloads.RekeyRequestPayload()
        b = payloads.RekeyRequestPayload()

        self.assertFalse(a != b)
        self.assertFalse(b != a)

        a = payloads.RekeyRequestPayload(
            unique_identifier='1346d253-69d6-474c-8cd5-ad475a3e0a81',
            offset=0,
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Activation Date'
                        ),
                        attribute_value=primitives.DateTime(
                            value=1136113200,
                            tag=enums.Tags.ACTIVATION_DATE
                        )
                    )
                ]
            )
        )
        b = payloads.RekeyRequestPayload(
            unique_identifier='1346d253-69d6-474c-8cd5-ad475a3e0a81',
            offset=0,
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Activation Date'
                        ),
                        attribute_value=primitives.DateTime(
                            value=1136113200,
                            tag=enums.Tags.ACTIVATION_DATE
                        )
                    )
                ]
            )
        )

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_not_equal_unique_identifier(self):
        """
        Test that the equality operator returns True when comparing two Rekey
        request payloads with different unique identifiers.
        """
        a = payloads.RekeyRequestPayload(
            unique_identifier='a'
        )
        b = payloads.RekeyRequestPayload(
            unique_identifier='b'
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_offset(self):
        """
        Test that the inequality operator returns True when comparing two Rekey
        request payloads with different offsets.
        """
        a = payloads.RekeyRequestPayload(
            offset=0
        )
        b = payloads.RekeyRequestPayload(
            offset=1
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_template_attribute(self):
        """
        Test that the inequality operator returns True when comparing two Rekey
        request payloads with different template attributes.
        """
        a = payloads.RekeyRequestPayload(
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Protect Stop Date'
                        ),
                        attribute_value=primitives.DateTime(
                            value=1577876400,
                            tag=enums.Tags.PROTECT_STOP_DATE
                        )
                    )
                ]
            )
        )
        b = payloads.RekeyRequestPayload(
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Deactivation Date'
                        ),
                        attribute_value=primitives.DateTime(
                            value=1577876400,
                            tag=enums.Tags.DEACTIVATION_DATE
                        )
                    )
                ]
            )
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_type_mismatch(self):
        """
        Test that the inequality operator returns True when comparing two Rekey
        request payloads with different types.
        """
        a = payloads.RekeyRequestPayload()
        b = 'invalid'

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_repr(self):
        """
        Test that repr can be applied to a Rekey request payload.
        """
        payload = payloads.RekeyRequestPayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            offset=0,
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Deactivation Date'
                        ),
                        attribute_value=primitives.DateTime(
                            value=1577876400,
                            tag=enums.Tags.DEACTIVATION_DATE
                        )
                    )
                ]
            )
        )

        # TODO (peter-hamilton) Update this when TemplateAttributes have repr
        expected = (
            "RekeyRequestPayload("
            "unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038', "
            "offset=0, "
            "template_attribute=Struct())"
        )
        observed = repr(payload)

        self.assertEqual(expected, observed)

    def test_str(self):
        """
        Test that str can be applied to a Rekey request payload
        """
        payload = payloads.RekeyRequestPayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            offset=0,
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Deactivation Date'
                        ),
                        attribute_value=primitives.DateTime(
                            value=1577876400,
                            tag=enums.Tags.DEACTIVATION_DATE
                        )
                    )
                ]
            )
        )

        # TODO (peter-hamilton) Update this when TemplateAttributes have str
        expected = str({
            'unique_identifier': '49a1ca88-6bea-4fb2-b450-7e58802c3038',
            'offset': 0,
            'template_attribute': 'Struct()'
        })
        observed = str(payload)

        self.assertEqual(expected, observed)


class TestRekeyResponsePayload(testtools.TestCase):
    """
    Test suite for the Rekey response payload.
    """

    def setUp(self):
        super(TestRekeyResponsePayload, self).setUp()

        # Encoding obtained from the KMIP 1.1 testing document,
        # Sections 3.1.1 and 9.2.
        #
        # This encoding matches the following set of values:
        # Response Payload
        #     Unique Identifier - 8efbbd67-2847-46b5-b7e7-4ab3b5e175de
        #     Template Attribute
        #         Attribute
        #             Attribute Name - Cryptographic Algorithm
        #             Attribute Value - AES
        #         Attribute
        #             Attribute Name - Cryptographic Length
        #             Attribute Value - 128

        self.full_encoding = utils.BytearrayStream(
            b'\x42\x00\x7C\x01\x00\x00\x00\xA8'
            b'\x42\x00\x94\x07\x00\x00\x00\x24'
            b'\x38\x65\x66\x62\x62\x64\x36\x37\x2D\x32\x38\x34\x37\x2D\x34\x36'
            b'\x62\x35\x2D\x62\x37\x65\x37\x2D\x34\x61\x62\x33\x62\x35\x65\x31'
            b'\x37\x35\x64\x65\x00\x00\x00\x00'
            b'\x42\x00\x91\x01\x00\x00\x00\x70'
            b'\x42\x00\x08\x01\x00\x00\x00\x30'
            b'\x42\x00\x0A\x07\x00\x00\x00\x17'
            b'\x43\x72\x79\x70\x74\x6F\x67\x72\x61\x70\x68\x69\x63\x20\x41\x6C'
            b'\x67\x6F\x72\x69\x74\x68\x6D\x00'
            b'\x42\x00\x0B\x05\x00\x00\x00\x04\x00\x00\x00\x03\x00\x00\x00\x00'
            b'\x42\x00\x08\x01\x00\x00\x00\x30'
            b'\x42\x00\x0A\x07\x00\x00\x00\x14'
            b'\x43\x72\x79\x70\x74\x6F\x67\x72\x61\x70\x68\x69\x63\x20\x4C\x65'
            b'\x6E\x67\x74\x68\x00\x00\x00\x00'
            b'\x42\x00\x0B\x02\x00\x00\x00\x04\x00\x00\x00\x80\x00\x00\x00\x00'
        )

        # Encoding obtained from the KMIP 1.1 testing document,
        # Sections 3.1.1 and 9.2.
        #
        # This encoding matches the following set of values:
        # Response Payload
        #     Unique Identifier - 8efbbd67-2847-46b5-b7e7-4ab3b5e175de

        self.partial_encoding = utils.BytearrayStream(
            b'\x42\x00\x7C\x01\x00\x00\x00\x30'
            b'\x42\x00\x94\x07\x00\x00\x00\x24'
            b'\x38\x65\x66\x62\x62\x64\x36\x37\x2D\x32\x38\x34\x37\x2D\x34\x36'
            b'\x62\x35\x2D\x62\x37\x65\x37\x2D\x34\x61\x62\x33\x62\x35\x65\x31'
            b'\x37\x35\x64\x65\x00\x00\x00\x00'
        )

        self.empty_encoding = utils.BytearrayStream(
            b'\x42\x00\x7C\x01\x00\x00\x00\x00'
        )

    def tearDown(self):
        super(TestRekeyResponsePayload, self).tearDown()

    def test_init(self):
        """
        Test that a Rekey response payload can be constructed with no
        arguments.
        """
        payload = payloads.RekeyResponsePayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.template_attribute)

    def test_init_with_args(self):
        """
        Test that a Rekey response payload can be constructed with valid
        values.
        """
        payload = payloads.RekeyResponsePayload(
            unique_identifier='00000000-2222-4444-6666-888888888888',
            template_attribute=objects.TemplateAttribute()
        )

        self.assertEqual(
            '00000000-2222-4444-6666-888888888888',
            payload.unique_identifier
        )
        self.assertEqual(
            objects.TemplateAttribute(),
            payload.template_attribute
        )

    def test_invalid_unique_identifier(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the unique identifier of a Rekey response payload.
        """
        kwargs = {'unique_identifier': 0}
        self.assertRaisesRegex(
            TypeError,
            "Unique identifier must be a string.",
            payloads.RekeyResponsePayload,
            **kwargs
        )

        args = (payloads.RekeyResponsePayload(), 'unique_identifier', 0)
        self.assertRaisesRegex(
            TypeError,
            "Unique identifier must be a string.",
            setattr,
            *args
        )

    def test_invalid_template_attribute(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the template attribute of a Rekey response payload.
        """
        kwargs = {'template_attribute': 'invalid'}
        self.assertRaisesRegex(
            TypeError,
            "Template attribute must be a TemplateAttribute struct.",
            payloads.RekeyResponsePayload,
            **kwargs
        )

        args = (
            payloads.RekeyResponsePayload(),
            'template_attribute',
            'invalid'
        )
        self.assertRaisesRegex(
            TypeError,
            "Template attribute must be a TemplateAttribute struct.",
            setattr,
            *args
        )

    def test_read(self):
        """
        Test that a Rekey response payload can be read from a data stream.
        """
        payload = payloads.RekeyResponsePayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.template_attribute)

        payload.read(self.full_encoding)

        self.assertEqual(
            '8efbbd67-2847-46b5-b7e7-4ab3b5e175de',
            payload.unique_identifier
        )
        self.assertEqual(
            objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Cryptographic Algorithm'
                        ),
                        attribute_value=primitives.Enumeration(
                            enums.CryptographicAlgorithm,
                            value=enums.CryptographicAlgorithm.AES,
                            tag=enums.Tags.CRYPTOGRAPHIC_ALGORITHM
                        )
                    ),
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Cryptographic Length'
                        ),
                        attribute_value=primitives.Integer(
                            value=128,
                            tag=enums.Tags.CRYPTOGRAPHIC_LENGTH
                        )
                    )
                ]
            ),
            payload.template_attribute
        )

    def test_read_partial(self):
        """
        Test that a Rekey response payload can be read from a partial data
        stream.
        """
        payload = payloads.RekeyResponsePayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.template_attribute)

        payload.read(self.partial_encoding)

        self.assertEqual(
            '8efbbd67-2847-46b5-b7e7-4ab3b5e175de',
            payload.unique_identifier
        )
        self.assertEqual(None, payload.template_attribute)

    def test_read_invalid(self):
        """
        Test that a ValueError gets raised when a required Rekey response
        payload attribute is missing from the payload encoding.
        """
        payload = payloads.RekeyResponsePayload()
        args = (self.empty_encoding, )
        self.assertRaisesRegex(
            ValueError,
            "The Rekey response payload encoding is missing the unique "
            "identifier.",
            payload.read,
            *args
        )

    def test_write(self):
        """
        Test that a Rekey response payload can be written to a data stream.
        """
        payload = payloads.RekeyResponsePayload(
            unique_identifier='8efbbd67-2847-46b5-b7e7-4ab3b5e175de',
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Cryptographic Algorithm'
                        ),
                        attribute_value=primitives.Enumeration(
                            enums.CryptographicAlgorithm,
                            value=enums.CryptographicAlgorithm.AES,
                            tag=enums.Tags.CRYPTOGRAPHIC_ALGORITHM
                        )
                    ),
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Cryptographic Length'
                        ),
                        attribute_value=primitives.Integer(
                            value=128,
                            tag=enums.Tags.CRYPTOGRAPHIC_LENGTH
                        )
                    )
                ]
            )
        )
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.full_encoding), len(stream))
        self.assertEqual(str(self.full_encoding), str(stream))

    def test_write_partial(self):
        """
        Test that a partial Rekey response payload can be written to a data
        stream.
        """
        payload = payloads.RekeyResponsePayload(
            unique_identifier='8efbbd67-2847-46b5-b7e7-4ab3b5e175de'
        )
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.partial_encoding), len(stream))
        self.assertEqual(str(self.partial_encoding), str(stream))

    def test_write_invalid(self):
        """
        Test that a ValueError gets raised when a required Rekey response
        payload attribute is missing when encoding the payload.
        """
        payload = payloads.RekeyResponsePayload()
        stream = utils.BytearrayStream()
        args = (stream, )
        self.assertRaisesRegex(
            ValueError,
            "The Rekey response payload is missing the unique identifier.",
            payload.write,
            *args
        )

    def test_equal_on_equal(self):
        """
        Test that the equality operator returns True when comparing two Rekey
        response payloads with the same data.
        """
        a = payloads.RekeyResponsePayload()
        b = payloads.RekeyResponsePayload()

        self.assertTrue(a == b)
        self.assertTrue(b == a)

        a = payloads.RekeyResponsePayload(
            unique_identifier='1346d253-69d6-474c-8cd5-ad475a3e0a81',
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Cryptographic Algorithm'
                        ),
                        attribute_value=primitives.Enumeration(
                            enums.CryptographicAlgorithm,
                            value=enums.CryptographicAlgorithm.AES,
                            tag=enums.Tags.CRYPTOGRAPHIC_ALGORITHM
                        )
                    ),
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Cryptographic Length'
                        ),
                        attribute_value=primitives.Integer(
                            value=128,
                            tag=enums.Tags.CRYPTOGRAPHIC_LENGTH
                        )
                    )
                ]
            )
        )
        b = payloads.RekeyResponsePayload(
            unique_identifier='1346d253-69d6-474c-8cd5-ad475a3e0a81',
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Cryptographic Algorithm'
                        ),
                        attribute_value=primitives.Enumeration(
                            enums.CryptographicAlgorithm,
                            value=enums.CryptographicAlgorithm.AES,
                            tag=enums.Tags.CRYPTOGRAPHIC_ALGORITHM
                        )
                    ),
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Cryptographic Length'
                        ),
                        attribute_value=primitives.Integer(
                            value=128,
                            tag=enums.Tags.CRYPTOGRAPHIC_LENGTH
                        )
                    )
                ]
            )
        )

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_not_equal_unique_identifier(self):
        """
        Test that the equality operator returns False when comparing two Rekey
        response payloads with different unique identifiers.
        """
        a = payloads.RekeyResponsePayload(
            unique_identifier='a'
        )
        b = payloads.RekeyResponsePayload(
            unique_identifier='b'
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_template_attribute(self):
        """
        Test that the equality operator returns False when comparing two Rekey
        response payloads with different template attributes.
        """
        a = payloads.RekeyResponsePayload(
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Cryptographic Algorithm'
                        ),
                        attribute_value=primitives.Enumeration(
                            enums.CryptographicAlgorithm,
                            value=enums.CryptographicAlgorithm.AES,
                            tag=enums.Tags.CRYPTOGRAPHIC_ALGORITHM
                        )
                    )
                ]
            )
        )
        b = payloads.RekeyResponsePayload(
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Cryptographic Length'
                        ),
                        attribute_value=primitives.Integer(
                            value=128,
                            tag=enums.Tags.CRYPTOGRAPHIC_LENGTH
                        )
                    )
                ]
            )
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_type_mismatch(self):
        """
        Test that the equality operator returns False when comparing two Rekey
        response payloads with different types.
        """
        a = payloads.RekeyResponsePayload()
        b = 'invalid'

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_not_equal_on_equal(self):
        """
        Test that the inequality operator returns False when comparing two
        Rekey response payloads with the same data.
        """
        a = payloads.RekeyResponsePayload()
        b = payloads.RekeyResponsePayload()

        self.assertFalse(a != b)
        self.assertFalse(b != a)

        a = payloads.RekeyResponsePayload(
            unique_identifier='1346d253-69d6-474c-8cd5-ad475a3e0a81',
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Cryptographic Algorithm'
                        ),
                        attribute_value=primitives.Enumeration(
                            enums.CryptographicAlgorithm,
                            value=enums.CryptographicAlgorithm.AES,
                            tag=enums.Tags.CRYPTOGRAPHIC_ALGORITHM
                        )
                    ),
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Cryptographic Length'
                        ),
                        attribute_value=primitives.Integer(
                            value=128,
                            tag=enums.Tags.CRYPTOGRAPHIC_LENGTH
                        )
                    )
                ]
            )
        )
        b = payloads.RekeyResponsePayload(
            unique_identifier='1346d253-69d6-474c-8cd5-ad475a3e0a81',
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Cryptographic Algorithm'
                        ),
                        attribute_value=primitives.Enumeration(
                            enums.CryptographicAlgorithm,
                            value=enums.CryptographicAlgorithm.AES,
                            tag=enums.Tags.CRYPTOGRAPHIC_ALGORITHM
                        )
                    ),
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Cryptographic Length'
                        ),
                        attribute_value=primitives.Integer(
                            value=128,
                            tag=enums.Tags.CRYPTOGRAPHIC_LENGTH
                        )
                    )
                ]
            )
        )

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_not_equal_unique_identifier(self):
        """
        Test that the inequality operator returns True when comparing two Rekey
        response payloads with different unique identifiers.
        """
        a = payloads.RekeyResponsePayload(
            unique_identifier='a'
        )
        b = payloads.RekeyResponsePayload(
            unique_identifier='b'
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_template_attribute(self):
        """
        Test that the inequality operator returns True when comparing two Rekey
        response payloads with different template attributes.
        """
        a = payloads.RekeyResponsePayload(
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Cryptographic Algorithm'
                        ),
                        attribute_value=primitives.Enumeration(
                            enums.CryptographicAlgorithm,
                            value=enums.CryptographicAlgorithm.AES,
                            tag=enums.Tags.CRYPTOGRAPHIC_ALGORITHM
                        )
                    )
                ]
            )
        )
        b = payloads.RekeyResponsePayload(
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Cryptographic Length'
                        ),
                        attribute_value=primitives.Integer(
                            value=128,
                            tag=enums.Tags.CRYPTOGRAPHIC_LENGTH
                        )
                    )
                ]
            )
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_type_mismatch(self):
        """
        Test that the inequality operator returns True when comparing two Rekey
        response payloads with different types.
        """
        a = payloads.RekeyResponsePayload()
        b = 'invalid'

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_repr(self):
        """
        Test that repr can be applied to a Rekey response payload.
        """
        payload = payloads.RekeyResponsePayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Cryptographic Length'
                        ),
                        attribute_value=primitives.Integer(
                            value=128,
                            tag=enums.Tags.CRYPTOGRAPHIC_LENGTH
                        )
                    )
                ]
            )
        )

        # TODO (peter-hamilton) Update this when TemplateAttributes have repr
        expected = (
            "RekeyResponsePayload("
            "unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038', "
            "template_attribute=Struct())"
        )
        observed = repr(payload)

        self.assertEqual(expected, observed)

    def test_str(self):
        """
        Test that str can be applied to a Rekey response payload
        """
        payload = payloads.RekeyResponsePayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            template_attribute=objects.TemplateAttribute(
                attributes=[
                    objects.Attribute(
                        attribute_name=objects.Attribute.AttributeName(
                            'Cryptographic Length'
                        ),
                        attribute_value=primitives.Integer(
                            value=128,
                            tag=enums.Tags.CRYPTOGRAPHIC_LENGTH
                        )
                    )
                ]
            )
        )

        # TODO (peter-hamilton) Update this when TemplateAttributes have str
        expected = str({
            'unique_identifier': '49a1ca88-6bea-4fb2-b450-7e58802c3038',
            'template_attribute': 'Struct()'
        })
        observed = str(payload)

        self.assertEqual(expected, observed)



# Copyright (c) 2018 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import requests
import testtools

from kmip.core import exceptions
from kmip.services.server import auth


class TestSLUGSConnector(testtools.TestCase):
    """
    Test suite for the SLUGSConnector.
    """

    def setUp(self):
        super(TestSLUGSConnector, self).setUp()

    def tearDown(self):
        super(TestSLUGSConnector, self).tearDown()

    def test_init(self):
        """
        Test that a SLUGSConnector can be constructed without arguments.
        """
        auth.SLUGSConnector()

    def test_init_with_args(self):
        """
        Test that a SLUGSConnector can be constructed with arguments.
        """
        connector = auth.SLUGSConnector(url='http://127.0.0.1:8080/slugs/')

        self.assertEqual('http://127.0.0.1:8080/slugs/', connector.url)
        self.assertEqual(
            'http://127.0.0.1:8080/slugs/users/{}',
            connector.users_url
        )
        self.assertEqual(
            'http://127.0.0.1:8080/slugs/users/{}/groups',
            connector.groups_url
        )

    def test_url_formatting(self):
        """
        Test that a URL without a trailing slash is handled properly when used
        to set the URL of a SLUGSConnector.
        """
        connector = auth.SLUGSConnector(url="http://127.0.0.1:8080/slugs")

        self.assertEqual('http://127.0.0.1:8080/slugs/', connector.url)
        self.assertEqual(
            'http://127.0.0.1:8080/slugs/users/{}',
            connector.users_url
        )
        self.assertEqual(
            'http://127.0.0.1:8080/slugs/users/{}/groups',
            connector.groups_url
        )

        connector = auth.SLUGSConnector()

        self.assertEqual(None, connector.url)
        self.assertEqual(None, connector.users_url)
        self.assertEqual(None, connector.groups_url)

        connector.url = "http://127.0.0.1:8080/slugs"

        self.assertEqual('http://127.0.0.1:8080/slugs/', connector.url)
        self.assertEqual(
            'http://127.0.0.1:8080/slugs/users/{}',
            connector.users_url
        )
        self.assertEqual(
            'http://127.0.0.1:8080/slugs/users/{}/groups',
            connector.groups_url
        )

    def test_invalid_url(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the URL of a SLUGSConnector.
        """
        kwargs = {'url': 0}
        self.assertRaisesRegex(
            TypeError,
            "URL must be a string.",
            auth.SLUGSConnector,
            **kwargs
        )

        connector = auth.SLUGSConnector()
        args = (connector, "url", 0)
        self.assertRaisesRegex(
            TypeError,
            "URL must be a string.",
            setattr,
            *args
        )

    @mock.patch('requests.get')
    @mock.patch(
        'kmip.services.server.auth.utils.get_client_identity_from_certificate'
    )
    def test_authenticate(self, mock_get_client_identity, mock_request_get):
        """
        Test that a call to authenticate with the SLUGSConnector triggers the
        right utility and SLUGS API calls.
        """
        mock_get_client_identity.return_value = "John Doe"

        users_response = mock.MagicMock(requests.Response)
        users_response.status_code = 200
        groups_response = mock.MagicMock(requests.Response)
        groups_response.status_code = 200
        groups_response.json.return_value = {'groups': ['Group A', 'Group B']}

        mock_request_get.side_effect = [users_response, groups_response]

        connector = auth.SLUGSConnector(
            url="http://127.0.0.1:8080/test/slugs/"
        )
        result = connector.authenticate("test")

        mock_get_client_identity.assert_called_once_with("test")
        mock_request_get.assert_any_call(
            "http://127.0.0.1:8080/test/slugs/users/John Doe"
        )
        mock_request_get.assert_any_call(
            "http://127.0.0.1:8080/test/slugs/users/John Doe/groups"
        )
        self.assertEqual(('John Doe', ['Group A', 'Group B']), result)

    @mock.patch('requests.get')
    @mock.patch(
        'kmip.services.server.auth.utils.get_client_identity_from_certificate'
    )
    def test_authenticate_with_url_unset(self,
                                         mock_get_client_identity,
                                         mock_request_get):
        """
        Test that a ConfigurationError is raised when attempting to
        authenticate with an unset URL.
        """
        connector = auth.SLUGSConnector()

        args = ("test", )
        self.assertRaisesRegex(
            exceptions.ConfigurationError,
            "The SLUGS URL must be specified.",
            connector.authenticate,
            *args
        )

    @mock.patch('requests.get')
    @mock.patch(
        'kmip.services.server.auth.utils.get_client_identity_from_certificate'
    )
    def test_authenticate_with_connection_failure(self,
                                                  mock_get_client_identity,
                                                  mock_request_get):
        """
        Test that a ConfigurationError is raised when attempting to
        authenticate with an invalid URL.
        """
        mock_get_client_identity.return_value = "John Doe"
        mock_request_get.side_effect = [requests.exceptions.ConnectionError()]

        connector = auth.SLUGSConnector(
            url="http://127.0.0.1:8080/test/slugs/"
        )
        args = ("test", )
        self.assertRaisesRegex(
            exceptions.ConfigurationError,
            "A connection could not be established using the SLUGS URL.",
            connector.authenticate,
            *args
        )

    @mock.patch('requests.get')
    @mock.patch(
        'kmip.services.server.auth.utils.get_client_identity_from_certificate'
    )
    def test_authenticate_with_users_failure(self,
                                             mock_get_client_identity,
                                             mock_request_get):
        """
        Test that a PermissionDenied error is raised when an invalid user ID
        is used to query SLUGS.
        """
        mock_get_client_identity.return_value = "John Doe"

        users_response = mock.MagicMock(requests.Response)
        users_response.status_code = 404

        mock_request_get.return_value = users_response

        connector = auth.SLUGSConnector(
            url="http://127.0.0.1:8080/test/slugs/"
        )
        args = ("test", )
        self.assertRaisesRegex(
            exceptions.PermissionDenied,
            "Unrecognized user ID: John Doe",
            connector.authenticate,
            *args
        )

    @mock.patch('requests.get')
    @mock.patch(
        'kmip.services.server.auth.utils.get_client_identity_from_certificate'
    )
    def test_authenticate_with_groups_failure(self,
                                              mock_get_client_identity,
                                              mock_request_get):
        """
        Test that a PermissionDenied error is raised when a groups request to
        SLUGS fails.
        """
        mock_get_client_identity.return_value = "John Doe"

        users_response = mock.MagicMock(requests.Response)
        users_response.status_code = 200
        groups_response = mock.MagicMock(requests.Response)
        groups_response.status_code = 404

        mock_request_get.side_effect = [users_response, groups_response]

        connector = auth.SLUGSConnector(
            url="http://127.0.0.1:8080/test/slugs/"
        )
        args = ("test", )
        self.assertRaisesRegex(
            exceptions.PermissionDenied,
            "Group information could not be retrieved for user ID: John Doe",
            connector.authenticate,
            *args
        )



# Copyright (c) 2019 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import binascii
import testtools
import sqlalchemy

from kmip.core import enums

from kmip.pie import objects
from kmip.pie import sqltypes


class TestSplitKey(testtools.TestCase):
    """
    Test suite for SplitKey.
    """

    def setUp(self):
        super(TestSplitKey, self).setUp()

        self.engine = sqlalchemy.create_engine("sqlite:///:memory:", echo=True)
        sqltypes.Base.metadata.create_all(self.engine)

    def tearDown(self):
        super(TestSplitKey, self).tearDown()

    def test_init(self):
        """
        Test that a SplitKey object can be instantiated.
        """
        split_key = objects.SplitKey()

        self.assertIsNone(split_key.cryptographic_algorithm)
        self.assertIsNone(split_key.cryptographic_length)
        self.assertIsNone(split_key.value)
        self.assertEqual(split_key.key_format_type, enums.KeyFormatType.RAW)
        self.assertEqual(split_key.cryptographic_usage_masks, [])
        self.assertEqual(split_key.names, ["Split Key"])
        self.assertIsNone(split_key.split_key_parts)
        self.assertIsNone(split_key.key_part_identifier)
        self.assertIsNone(split_key.split_key_threshold)
        self.assertIsNone(split_key.split_key_method)
        self.assertIsNone(split_key.prime_field_size)

    def test_init_with_args(self):
        """
        Test that a SplitKey object can be instantiated with all arguments.
        """
        split_key = objects.SplitKey(
            cryptographic_algorithm=enums.CryptographicAlgorithm.AES,
            cryptographic_length=128,
            key_value=(
                b'\x66\xC4\x6A\x77\x54\xF9\x4D\xE4'
                b'\x20\xC7\xB1\xA7\xFF\xF5\xEC\x56'
            ),
            key_format_type=enums.KeyFormatType.RAW,
            cryptographic_usage_masks=[
                enums.CryptographicUsageMask.EXPORT
            ],
            name="Test Split Key",
            split_key_parts=4,
            key_part_identifier=1,
            split_key_threshold=2,
            split_key_method=enums.SplitKeyMethod.POLYNOMIAL_SHARING_GF_2_8,
            prime_field_size=104729
        )

        self.assertEqual(
            split_key.cryptographic_algorithm,
            enums.CryptographicAlgorithm.AES
        )
        self.assertEqual(split_key.cryptographic_length, 128)
        self.assertEqual(
            split_key.value,
            b'\x66\xC4\x6A\x77\x54\xF9\x4D\xE4\x20\xC7\xB1\xA7\xFF\xF5\xEC\x56'
        )
        self.assertEqual(split_key.key_format_type, enums.KeyFormatType.RAW)
        self.assertEqual(
            split_key.cryptographic_usage_masks,
            [enums.CryptographicUsageMask.EXPORT]
        )
        self.assertEqual(split_key.names, ["Test Split Key"])
        self.assertEqual(split_key.split_key_parts, 4)
        self.assertEqual(split_key.key_part_identifier, 1)
        self.assertEqual(split_key.split_key_threshold, 2)
        self.assertEqual(
            split_key.split_key_method,
            enums.SplitKeyMethod.POLYNOMIAL_SHARING_GF_2_8
        )
        self.assertEqual(split_key.prime_field_size, 104729)

    def test_invalid_split_key_parts(self):
        """
        Test that a TypeError is raised when an invalid split key parts value
        is used to construct a SplitKey.
        """
        kwargs = {"split_key_parts": "invalid"}

        self.assertRaisesRegex(
            TypeError,
            "The split key parts must be an integer.",
            objects.SplitKey,
            **kwargs
        )

        args = (
            objects.SplitKey(),
            "split_key_parts",
            "invalid"
        )
        self.assertRaisesRegex(
            TypeError,
            "The split key parts must be an integer.",
            setattr,
            *args
        )

    def test_invalid_key_part_identifier(self):
        """
        Test that a TypeError is raised when an invalid key part identifier
        value is used to construct a SplitKey.
        """
        kwargs = {"key_part_identifier": "invalid"}

        self.assertRaisesRegex(
            TypeError,
            "The key part identifier must be an integer.",
            objects.SplitKey,
            **kwargs
        )

        args = (
            objects.SplitKey(),
            "key_part_identifier",
            "invalid"
        )
        self.assertRaisesRegex(
            TypeError,
            "The key part identifier must be an integer.",
            setattr,
            *args
        )

    def test_invalid_split_key_threshold(self):
        """
        Test that a TypeError is raised when an invalid split key threshold
        value is used to construct a SplitKey.
        """
        kwargs = {"split_key_threshold": "invalid"}

        self.assertRaisesRegex(
            TypeError,
            "The split key threshold must be an integer.",
            objects.SplitKey,
            **kwargs
        )

        args = (
            objects.SplitKey(),
            "split_key_threshold",
            "invalid"
        )
        self.assertRaisesRegex(
            TypeError,
            "The split key threshold must be an integer.",
            setattr,
            *args
        )

    def test_invalid_split_key_method(self):
        """
        Test that a TypeError is raised when an invalid split key method value
        is used to construct a SplitKey.
        """
        kwargs = {"split_key_method": "invalid"}

        self.assertRaisesRegex(
            TypeError,
            "The split key method must be a SplitKeyMethod enumeration.",
            objects.SplitKey,
            **kwargs
        )

        args = (
            objects.SplitKey(),
            "split_key_method",
            "invalid"
        )
        self.assertRaisesRegex(
            TypeError,
            "The split key method must be a SplitKeyMethod enumeration.",
            setattr,
            *args
        )

    def test_invalid_prime_field_size(self):
        """
        Test that a TypeError is raised when an invalid prime field size value
        is used to construct a SplitKey.
        """
        kwargs = {"prime_field_size": "invalid"}

        self.assertRaisesRegex(
            TypeError,
            "The prime field size must be an integer.",
            objects.SplitKey,
            **kwargs
        )

        args = (
            objects.SplitKey(),
            "prime_field_size",
            "invalid"
        )
        self.assertRaisesRegex(
            TypeError,
            "The prime field size must be an integer.",
            setattr,
            *args
        )

    def test_repr(self):
        """
        Test that repr can be applied to a SplitKey.
        """
        split_key = objects.SplitKey(
            cryptographic_algorithm=enums.CryptographicAlgorithm.AES,
            cryptographic_length=128,
            key_value=(
                b'\x66\xC4\x6A\x77\x54\xF9\x4D\xE4'
                b'\x20\xC7\xB1\xA7\xFF\xF5\xEC\x56'
            ),
            key_format_type=enums.KeyFormatType.RAW,
            cryptographic_usage_masks=[
                enums.CryptographicUsageMask.EXPORT
            ],
            name="Test Split Key",
            split_key_parts=4,
            key_part_identifier=1,
            split_key_threshold=2,
            split_key_method=enums.SplitKeyMethod.POLYNOMIAL_SHARING_GF_2_8,
            prime_field_size=104729
        )

        args = [
            "cryptographic_algorithm={}".format(
                enums.CryptographicAlgorithm.AES
            ),
            "cryptographic_length={}".format(128),
            "key_value={}".format(
                binascii.hexlify(
                    b'\x66\xC4\x6A\x77\x54\xF9\x4D\xE4'
                    b'\x20\xC7\xB1\xA7\xFF\xF5\xEC\x56'
                )
            ),
            "key_format_type={}".format(enums.KeyFormatType.RAW),
            "key_wrapping_data={}".format({}),
            "cryptographic_usage_masks={}".format(
                [enums.CryptographicUsageMask.EXPORT]
            ),
            "name={}".format(["Test Split Key"]),
            "split_key_parts=4",
            "key_part_identifier=1",
            "split_key_threshold=2",
            "split_key_method={}".format(
                enums.SplitKeyMethod.POLYNOMIAL_SHARING_GF_2_8
            ),
            "prime_field_size=104729"
        ]

        expected = "SplitKey({})".format(", ".join(args))
        observed = repr(split_key)

        self.assertEqual(expected, observed)

    def test_str(self):
        """
        Test that str can be applied to a SplitKey.
        """
        split_key = objects.SplitKey(
            key_value=(
                b'\x66\xC4\x6A\x77\x54\xF9\x4D\xE4'
                b'\x20\xC7\xB1\xA7\xFF\xF5\xEC\x56'
            )
        )

        expected = str(binascii.hexlify(split_key.value))
        observed = str(split_key)

        self.assertEqual(expected, observed)

    def test_comparison_on_equal(self):
        """
        Test that the equality/inequality operators return True/False when
        comparing two SplitKey objects with the same data.
        """
        a = objects.SplitKey()
        b = objects.SplitKey()

        self.assertTrue(a == b)
        self.assertTrue(b == a)
        self.assertFalse(a != b)
        self.assertFalse(b != a)

        a = objects.SplitKey(
            cryptographic_algorithm=enums.CryptographicAlgorithm.AES,
            cryptographic_length=128,
            key_value=(
                b'\x66\xC4\x6A\x77\x54\xF9\x4D\xE4'
                b'\x20\xC7\xB1\xA7\xFF\xF5\xEC\x56'
            ),
            key_format_type=enums.KeyFormatType.RAW,
            cryptographic_usage_masks=[
                enums.CryptographicUsageMask.EXPORT
            ],
            name="Test Split Key",
            split_key_parts=4,
            key_part_identifier=1,
            split_key_threshold=2,
            split_key_method=enums.SplitKeyMethod.POLYNOMIAL_SHARING_GF_2_8,
            prime_field_size=104729
        )
        b = objects.SplitKey(
            cryptographic_algorithm=enums.CryptographicAlgorithm.AES,
            cryptographic_length=128,
            key_value=(
                b'\x66\xC4\x6A\x77\x54\xF9\x4D\xE4'
                b'\x20\xC7\xB1\xA7\xFF\xF5\xEC\x56'
            ),
            key_format_type=enums.KeyFormatType.RAW,
            cryptographic_usage_masks=[
                enums.CryptographicUsageMask.EXPORT
            ],
            name="Test Split Key",
            split_key_parts=4,
            key_part_identifier=1,
            split_key_threshold=2,
            split_key_method=enums.SplitKeyMethod.POLYNOMIAL_SHARING_GF_2_8,
            prime_field_size=104729
        )

        self.assertTrue(a == b)
        self.assertTrue(b == a)
        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_comparison_on_different_cryptographic_algorithms(self):
        """
        Test that the equality/inequality operators return False/True when
        comparing two SplitKey objects with different cryptographic algorithms.
        """
        a = objects.SplitKey(
            cryptographic_algorithm=enums.CryptographicAlgorithm.AES
        )
        b = objects.SplitKey(
            cryptographic_algorithm=enums.CryptographicAlgorithm.RSA
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_comparison_on_different_cryptographic_lengths(self):
        """
        Test that the equality/inequality operators return False/True when
        comparing two SplitKey objects with different cryptographic lengths.
        """
        a = objects.SplitKey(cryptographic_length=128)
        b = objects.SplitKey(cryptographic_length=256)

        self.assertFalse(a == b)
        self.assertFalse(b == a)
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_comparison_on_different_values(self):
        """
        Test that the equality/inequality operators return False/True when
        comparing two SplitKey objects with different values.
        """
        a = objects.SplitKey(key_value=b'\x00')
        b = objects.SplitKey(key_value=b'\xFF')

        self.assertFalse(a == b)
        self.assertFalse(b == a)
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_comparison_on_different_key_format_types(self):
        """
        Test that the equality/inequality operators return False/True when
        comparing two SplitKey objects with different key format types.
        """
        a = objects.SplitKey(key_format_type=enums.KeyFormatType.RAW)
        b = objects.SplitKey(key_format_type=enums.KeyFormatType.OPAQUE)

        self.assertFalse(a == b)
        self.assertFalse(b == a)
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_comparison_on_different_key_wrapping_data(self):
        """
        Test that the equality/inequality operators return False/True when
        comparing two SplitKey objects with different key wrapping data.
        """
        a = objects.SplitKey(key_wrapping_data={})
        b = objects.SplitKey(
            key_wrapping_data={"wrapping_method": enums.WrappingMethod.ENCRYPT}
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_comparison_on_different_cryptographic_usage_masks(self):
        """
        Test that the equality/inequality operators return False/True when
        comparing two SplitKey objects with different cryptographic usage
        masks.
        """
        a = objects.SplitKey(
            cryptographic_usage_masks=[enums.CryptographicUsageMask.ENCRYPT]
        )
        b = objects.SplitKey(
            cryptographic_usage_masks=[enums.CryptographicUsageMask.EXPORT]
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_comparison_on_different_names(self):
        """
        Test that the equality/inequality operators return False/True when
        comparing two SplitKey objects with different names.
        """
        a = objects.SplitKey(name="Test Split Key")
        b = objects.SplitKey(name="Split Key Test")

        self.assertFalse(a == b)
        self.assertFalse(b == a)
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_comparison_on_different_split_key_parts(self):
        """
        Test that the equality/inequality operators return False/True when
        comparing two SplitKey objects with different split key parts.
        """
        a = objects.SplitKey(split_key_parts=4)
        b = objects.SplitKey(split_key_parts=5)

        self.assertFalse(a == b)
        self.assertFalse(b == a)
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_comparison_on_different_key_part_identifiers(self):
        """
        Test that the equality/inequality operators return False/True when
        comparing two SplitKey objects with different key part identifiers.
        """
        a = objects.SplitKey(key_part_identifier=1)
        b = objects.SplitKey(key_part_identifier=2)

        self.assertFalse(a == b)
        self.assertFalse(b == a)
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_comparison_on_different_split_key_thresholds(self):
        """
        Test that the equality/inequality operators return False/True when
        comparing two SplitKey objects with different split key thresholds.
        """
        a = objects.SplitKey(split_key_threshold=1)
        b = objects.SplitKey(split_key_threshold=2)

        self.assertFalse(a == b)
        self.assertFalse(b == a)
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_comparison_on_different_split_key_methods(self):
        """
        Test that the equality/inequality operators return False/True when
        comparing two SplitKey objects with different split key methods.
        """
        a = objects.SplitKey(split_key_method=enums.SplitKeyMethod.XOR)
        b = objects.SplitKey(
            split_key_method=enums.SplitKeyMethod.POLYNOMIAL_SHARING_GF_2_8
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_comparison_on_different_prime_field_sizes(self):
        """
        Test that the equality/inequality operators return False/True when
        comparing two SplitKey objects with different prime field sizes.
        """
        a = objects.SplitKey(prime_field_size=13)
        b = objects.SplitKey(prime_field_size=104729)

        self.assertFalse(a == b)
        self.assertFalse(b == a)
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_comparison_on_type_mismatch(self):
        """
        Test that the equality/inequality operators return False/True when
        comparing a SplitKey object to a non-SplitKey object.
        """
        a = objects.SplitKey()
        b = "invalid"

        self.assertFalse(a == b)
        self.assertFalse(b == a)
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_save(self):
        """
        Test that a SplitKey object can be saved using SQLAlchemy. This will
        add it to the database, verify that no exceptions are thrown, and check
        that its unique identifier was set.
        """
        split_key = objects.SplitKey(
            cryptographic_algorithm=enums.CryptographicAlgorithm.AES,
            cryptographic_length=128,
            key_value=(
                b'\x66\xC4\x6A\x77\x54\xF9\x4D\xE4'
                b'\x20\xC7\xB1\xA7\xFF\xF5\xEC\x56'
            ),
            key_format_type=enums.KeyFormatType.RAW,
            cryptographic_usage_masks=[
                enums.CryptographicUsageMask.EXPORT
            ],
            name="Test Split Key",
            split_key_parts=4,
            key_part_identifier=1,
            split_key_threshold=2,
            split_key_method=enums.SplitKeyMethod.POLYNOMIAL_SHARING_GF_2_8,
            prime_field_size=104729
        )

        session = sqlalchemy.orm.sessionmaker(bind=self.engine)()
        session.add(split_key)
        session.commit()

        self.assertIsNotNone(split_key.unique_identifier)

    def test_get(self):
        """
        Test that a SplitKey object can be saved and then retrieved using
        SQLAlchemy. This test adds the object to the database and then
        retrieves it by ID and verifies some of the attributes.
        """
        split_key = objects.SplitKey(
            cryptographic_algorithm=enums.CryptographicAlgorithm.AES,
            cryptographic_length=128,
            key_value=(
                b'\x66\xC4\x6A\x77\x54\xF9\x4D\xE4'
                b'\x20\xC7\xB1\xA7\xFF\xF5\xEC\x56'
            ),
            key_format_type=enums.KeyFormatType.RAW,
            cryptographic_usage_masks=[
                enums.CryptographicUsageMask.EXPORT
            ],
            name="Test Split Key",
            split_key_parts=4,
            key_part_identifier=1,
            split_key_threshold=2,
            split_key_method=enums.SplitKeyMethod.POLYNOMIAL_SHARING_GF_2_8,
            prime_field_size=104729
        )

        session = sqlalchemy.orm.sessionmaker(bind=self.engine)()
        session.add(split_key)
        session.commit()

        session = sqlalchemy.orm.sessionmaker(bind=self.engine)()
        retrieved_key = session.query(objects.SplitKey).filter(
            objects.ManagedObject.unique_identifier ==
            split_key.unique_identifier
        ).one()
        session.commit()

        self.assertEqual(retrieved_key.names, ["Test Split Key"])
        self.assertEqual(
            retrieved_key.cryptographic_algorithm,
            enums.CryptographicAlgorithm.AES
        )
        self.assertEqual(retrieved_key.cryptographic_length, 128)
        self.assertEqual(
            retrieved_key.value,
            (
                b'\x66\xC4\x6A\x77\x54\xF9\x4D\xE4'
                b'\x20\xC7\xB1\xA7\xFF\xF5\xEC\x56'
            )
        )
        self.assertEqual(
            retrieved_key.key_format_type,
            enums.KeyFormatType.RAW
        )
        self.assertEqual(
            retrieved_key.cryptographic_usage_masks,
            [enums.CryptographicUsageMask.EXPORT]
        )
        self.assertEqual(retrieved_key.split_key_parts, 4)
        self.assertEqual(retrieved_key.key_part_identifier, 1)
        self.assertEqual(retrieved_key.split_key_threshold, 2)
        self.assertEqual(
            retrieved_key.split_key_method,
            enums.SplitKeyMethod.POLYNOMIAL_SHARING_GF_2_8
        )
        self.assertEqual(retrieved_key.prime_field_size, 104729)



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from kmip.core import exceptions
from kmip.core import primitives
from kmip.core import utils


class TestLongInteger(testtools.TestCase):
    """
    Test suite for the LongInteger primitive.
    """

    def setUp(self):
        super(TestLongInteger, self).setUp()

    def tearDown(self):
        super(TestLongInteger, self).tearDown()

    def test_init(self):
        """
        Test that a LongInteger can be instantiated.
        """
        long_int = primitives.LongInteger(1)
        self.assertEqual(1, long_int.value)

    def test_init_unset(self):
        """
        Test that a LongInteger can be instantiated with no input.
        """
        long_int = primitives.LongInteger()
        self.assertEqual(0, long_int.value)

    def test_init_on_max(self):
        """
        Test that a LongInteger can be instantiated with the maximum possible
        signed 64-bit value.
        """
        primitives.LongInteger(primitives.LongInteger.MAX)

    def test_init_on_min(self):
        """
        Test that a LongInteger can be instantiated with the minimum possible
        signed 64-bit value.
        """
        primitives.LongInteger(primitives.LongInteger.MIN)

    def test_validate_on_valid(self):
        """
        Test that a LongInteger can be validated on good input.
        """
        long_int = primitives.LongInteger(1)
        long_int.validate()

    def test_validate_on_valid_unset(self):
        """
        Test that a LongInteger with no preset value can be validated.
        """
        long_int = primitives.LongInteger()
        long_int.validate()

    def test_validate_on_invalid_type(self):
        """
        Test that a TypeError is thrown on input of invalid type (e.g., str).
        """
        self.assertRaises(TypeError, primitives.LongInteger, 'invalid')

    def test_validate_on_invalid_value_too_big(self):
        """
        Test that a ValueError is thrown on input that is too large.
        """
        self.assertRaises(
            ValueError, primitives.LongInteger, primitives.LongInteger.MAX + 1)

    def test_validate_on_invalid_value_too_small(self):
        """
        Test that a ValueError is thrown on input that is too small.
        """
        self.assertRaises(
            ValueError, primitives.LongInteger, primitives.LongInteger.MIN - 1)

    def test_read_zero(self):
        """
        Test that a LongInteger representing the value 0 can be read from a
        byte stream.
        """
        encoding = (
            b'\x42\x00\x00\x03\x00\x00\x00\x08\x00\x00\x00\x00\x00\x00\x00'
            b'\x00')
        stream = utils.BytearrayStream(encoding)
        long_int = primitives.LongInteger()
        long_int.read(stream)
        self.assertEqual(0, long_int.value)

    def test_read_max_max(self):
        """
        Test that a LongInteger representing the maximum positive value can be
        read from a byte stream.
        """
        encoding = (
            b'\x42\x00\x00\x03\x00\x00\x00\x08\x7f\xff\xff\xff\xff\xff\xff'
            b'\xff')
        stream = utils.BytearrayStream(encoding)
        long_int = primitives.LongInteger()
        long_int.read(stream)
        self.assertEqual(primitives.LongInteger.MAX, long_int.value)

    def test_read_min_max(self):
        """
        Test that a LongInteger representing the minimum positive value can be
        read from a byte stream.
        """
        encoding = (
            b'\x42\x00\x00\x03\x00\x00\x00\x08\x00\x00\x00\x00\x00\x00\x00'
            b'\x01')
        stream = utils.BytearrayStream(encoding)
        long_int = primitives.LongInteger()
        long_int.read(stream)
        self.assertEqual(1, long_int.value)

    def test_read_max_min(self):
        """
        Test that a LongInteger representing the maximum negative value can be
        read from a byte stream.
        """
        encoding = (
            b'\x42\x00\x00\x03\x00\x00\x00\x08\xff\xff\xff\xff\xff\xff\xff'
            b'\xff')
        stream = utils.BytearrayStream(encoding)
        long_int = primitives.LongInteger()
        long_int.read(stream)
        self.assertEqual(-1, long_int.value)

    def test_read_min_min(self):
        """
        Test that a LongInteger representing the minimum negative value can be
        read from a byte stream.
        """
        encoding = (
            b'\x42\x00\x00\x03\x00\x00\x00\x08\x80\x00\x00\x00\x00\x00\x00'
            b'\x00')
        stream = utils.BytearrayStream(encoding)
        long_int = primitives.LongInteger(primitives.LongInteger.MIN)
        long_int.read(stream)
        self.assertEqual(primitives.LongInteger.MIN, long_int.value)

    def test_read_on_invalid_length(self):
        """
        Test that an InvalidPrimitiveLength exception is thrown when attempting
        to decode a LongInteger with an invalid length.
        """
        encoding = (
            b'\x42\x00\x00\x03\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
            b'\x00')
        stream = utils.BytearrayStream(encoding)
        long_int = primitives.LongInteger()

        self.assertRaises(
            exceptions.InvalidPrimitiveLength, long_int.read, stream)

    def test_write_zero(self):
        """
        Test that a LongInteger representing the value 0 can be written to a
        byte stream.
        """
        encoding = (
            b'\x42\x00\x00\x03\x00\x00\x00\x08\x00\x00\x00\x00\x00\x00\x00'
            b'\x00')
        stream = utils.BytearrayStream()
        long_int = primitives.LongInteger(0)
        long_int.write(stream)

        result = stream.read()
        self.assertEqual(len(encoding), len(result))
        self.assertEqual(encoding, result)

    def test_write_max_max(self):
        """
        Test that a LongInteger representing the maximum positive value can be
        written to a byte stream.
        """
        encoding = (
            b'\x42\x00\x00\x03\x00\x00\x00\x08\x7f\xff\xff\xff\xff\xff\xff'
            b'\xff')
        stream = utils.BytearrayStream()
        long_int = primitives.LongInteger(primitives.LongInteger.MAX)
        long_int.write(stream)

        result = stream.read()
        self.assertEqual(len(encoding), len(result))
        self.assertEqual(encoding, result)

    def test_write_min_max(self):
        """
        Test that a LongInteger representing the minimum positive value can be
        written to a byte stream.
        """
        encoding = (
            b'\x42\x00\x00\x03\x00\x00\x00\x08\x00\x00\x00\x00\x00\x00\x00'
            b'\x01')
        stream = utils.BytearrayStream()
        long_int = primitives.LongInteger(1)
        long_int.write(stream)

        result = stream.read()
        self.assertEqual(len(encoding), len(result))
        self.assertEqual(encoding, result)

    def test_write_max_min(self):
        """
        Test that a LongInteger representing the maximum negative value can be
        written to a byte stream.
        """
        encoding = (
            b'\x42\x00\x00\x03\x00\x00\x00\x08\xff\xff\xff\xff\xff\xff\xff'
            b'\xff')
        stream = utils.BytearrayStream()
        long_int = primitives.LongInteger(-1)
        long_int.write(stream)

        result = stream.read()
        self.assertEqual(len(encoding), len(result))
        self.assertEqual(encoding, result)

    def test_write_min_min(self):
        """
        Test that a LongInteger representing the minimum negative value can be
        written to a byte stream.
        """
        encoding = (
            b'\x42\x00\x00\x03\x00\x00\x00\x08\x80\x00\x00\x00\x00\x00\x00'
            b'\x00')
        stream = utils.BytearrayStream()
        long_int = primitives.LongInteger(primitives.LongInteger.MIN)
        long_int.write(stream)

        result = stream.read()
        self.assertEqual(len(encoding), len(result))
        self.assertEqual(encoding, result)

    def test_repr(self):
        """
        Test that the representation of a LongInteger is formatted properly.
        """
        long_int = primitives.LongInteger()
        value = "value={0}".format(long_int.value)
        tag = "tag={0}".format(long_int.tag)
        self.assertEqual(
            "LongInteger({0}, {1})".format(value, tag), repr(long_int))

    def test_str(self):
        """
        Test that the string representation of a LongInteger is formatted
        properly.
        """
        self.assertEqual("0", str(primitives.LongInteger()))

    def test_equal_on_equal(self):
        """
        Test that the equality operator returns True when comparing two
        LongIntegers.
        """
        a = primitives.LongInteger(1)
        b = primitives.LongInteger(1)

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_equal_and_empty(self):
        """
        Test that the equality operator returns True when comparing two
        LongIntegers.
        """
        a = primitives.LongInteger()
        b = primitives.LongInteger()

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_not_equal(self):
        """
        Test that the equality operator returns False when comparing two
        LongIntegers with different values.
        """
        a = primitives.LongInteger(1)
        b = primitives.LongInteger(2)

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_type_mismatch(self):
        """
        Test that the equality operator returns False when comparing a
        LongInteger to a non-LongInteger object.
        """
        a = primitives.LongInteger()
        b = 'invalid'

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_not_equal_on_equal(self):
        """
        Test that the inequality operator returns False when comparing
        two LongIntegers with the same values.
        """
        a = primitives.LongInteger(1)
        b = primitives.LongInteger(1)

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_equal_and_empty(self):
        """
        Test that the inequality operator returns False when comparing
        two LongIntegers.
        """
        a = primitives.LongInteger()
        b = primitives.LongInteger()

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_not_equal(self):
        """
        Test that the inequality operator returns True when comparing two
        LongIntegers with different values.
        """
        a = primitives.LongInteger(1)
        b = primitives.LongInteger(2)

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_type_mismatch(self):
        """
        Test that the inequality operator returns True when comparing a
        LongInteger to a non-LongInteger object.
        """
        a = primitives.LongInteger()
        b = 'invalid'

        self.assertTrue(a != b)
        self.assertTrue(b != a)



# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from kmip.core import attributes
from kmip.core import enums
from kmip.core import primitives

from kmip.core.factories import attribute_values


class TestAttributeValueFactory(testtools.TestCase):

    def setUp(self):
        super(TestAttributeValueFactory, self).setUp()
        self.factory = attribute_values.AttributeValueFactory()

    def tearDown(self):
        super(TestAttributeValueFactory, self).tearDown()

    def test_create_unique_identifier(self):
        """
        Test that a UniqueIdentifier attribute can be created.
        """
        self.skipTest('')

    def test_create_name(self):
        """
        Test that a Name attribute can be created.
        """
        attr_type = enums.AttributeType.NAME
        name = self.factory.create_attribute_value(attr_type, "foo")
        self.assertIsInstance(name, attributes.Name)
        self.assertEqual("foo", name.name_value.value)
        self.assertEqual(enums.Tags.NAME, name.tag)

    def test_create_object_type(self):
        """
        Test that an empty ObjectType attribute can be created.
        """
        object_type = self.factory.create_attribute_value(
            enums.AttributeType.OBJECT_TYPE,
            None
        )
        self.assertIsInstance(object_type, attributes.ObjectType)
        self.assertEqual(None, object_type.value)

    def test_create_object_type_with_value(self):
        """
        Test that an ObjectType attribute can be created with a custom value.
        """
        object_type = self.factory.create_attribute_value(
            enums.AttributeType.OBJECT_TYPE,
            enums.ObjectType.SYMMETRIC_KEY
        )
        self.assertIsInstance(object_type, attributes.ObjectType)
        self.assertEqual(enums.ObjectType.SYMMETRIC_KEY, object_type.value)

    def test_create_cryptographic_algorithm(self):
        """
        Test that a CryptographicAlgorithm attribute can be created.
        """
        self.skipTest('')

    def test_create_cryptographic_length(self):
        """
        Test that a CryptographicLength attribute can be created.
        """
        self.skipTest('')

    def test_create_cryptographic_parameters(self):
        """
        Test that a CryptographicParameters attribute can be created.
        """
        value = {
            'block_cipher_mode': enums.BlockCipherMode.NIST_KEY_WRAP,
            'padding_method': enums.PaddingMethod.ANSI_X923,
            'key_role_type': enums.KeyRoleType.KEK,
            'hashing_algorithm': enums.HashingAlgorithm.SHA_512,
            'digital_signature_algorithm':
                enums.DigitalSignatureAlgorithm.ECDSA_WITH_SHA512,
            'cryptographic_algorithm':
                enums.CryptographicAlgorithm.HMAC_SHA512,
            'random_iv': True,
            'iv_length': 96,
            'tag_length': None,
            'fixed_field_length': 32,
            'invocation_field_length': 64,
            'counter_length': None,
            'initial_counter_value': 1
        }
        cryptographic_parameters = self.factory.create_attribute_value(
            enums.AttributeType.CRYPTOGRAPHIC_PARAMETERS,
            value
        )

        self.assertIsInstance(
            cryptographic_parameters,
            attributes.CryptographicParameters
        )
        self.assertEqual(
            enums.BlockCipherMode.NIST_KEY_WRAP,
            cryptographic_parameters.block_cipher_mode
        )
        self.assertEqual(
            enums.PaddingMethod.ANSI_X923,
            cryptographic_parameters.padding_method
        )
        self.assertEqual(
            enums.KeyRoleType.KEK,
            cryptographic_parameters.key_role_type
        )
        self.assertEqual(
            enums.HashingAlgorithm.SHA_512,
            cryptographic_parameters.hashing_algorithm
        )
        self.assertEqual(
            enums.DigitalSignatureAlgorithm.ECDSA_WITH_SHA512,
            cryptographic_parameters.digital_signature_algorithm
        )
        self.assertEqual(
            enums.CryptographicAlgorithm.HMAC_SHA512,
            cryptographic_parameters.cryptographic_algorithm
        )
        self.assertEqual(True, cryptographic_parameters.random_iv)
        self.assertEqual(96, cryptographic_parameters.iv_length)
        self.assertEqual(None, cryptographic_parameters.tag_length)
        self.assertEqual(32, cryptographic_parameters.fixed_field_length)
        self.assertEqual(64, cryptographic_parameters.invocation_field_length)
        self.assertEqual(None, cryptographic_parameters.counter_length)
        self.assertEqual(1, cryptographic_parameters.initial_counter_value)

    def test_create_cryptographic_domain_parameters(self):
        """
        Test that a CryptographicDomainParameters attribute can be created.
        """
        kwargs = {'name': enums.AttributeType.CRYPTOGRAPHIC_DOMAIN_PARAMETERS,
                  'value': None}
        self.assertRaises(
            NotImplementedError, self.factory.create_attribute_value, **kwargs)

    def test_create_certificate_type(self):
        """
        Test that a CertificateType attribute can be created.
        """
        certificate_type = self.factory.create_attribute_value(
            name=enums.AttributeType.CERTIFICATE_TYPE,
            value=enums.CertificateType.X_509
        )
        self.assertIsInstance(certificate_type, primitives.Enumeration)
        self.assertEqual(enums.CertificateType.X_509, certificate_type.value)
        self.assertEqual(enums.Tags.CERTIFICATE_TYPE, certificate_type.tag)

    def test_create_certificate_length(self):
        """
        Test that a CertificateLength attribute can be created.
        """
        length = self.factory.create_attribute_value(
            enums.AttributeType.CERTIFICATE_LENGTH, 0)
        self.assertIsInstance(length, primitives.Integer)
        self.assertEqual(0, length.value)
        self.assertEqual(enums.Tags.CERTIFICATE_LENGTH, length.tag)

    def test_create_x509_certificate_identifier(self):
        """
        Test that an X509CertificateIdentifier attribute can be created.
        """
        kwargs = {'name': enums.AttributeType.X_509_CERTIFICATE_IDENTIFIER,
                  'value': None}
        self.assertRaises(
            NotImplementedError, self.factory.create_attribute_value, **kwargs)

    def test_create_x509_certificate_subject(self):
        """
        Test that an X509CertificateSubject attribute can be created.
        """
        kwargs = {'name': enums.AttributeType.X_509_CERTIFICATE_SUBJECT,
                  'value': None}
        self.assertRaises(
            NotImplementedError, self.factory.create_attribute_value, **kwargs)

    def test_create_x509_certificate_issuer(self):
        """
        Test that an X509CertificateIssuer attribute can be created.
        """
        kwargs = {'name': enums.AttributeType.X_509_CERTIFICATE_ISSUER,
                  'value': None}
        self.assertRaises(
            NotImplementedError, self.factory.create_attribute_value, **kwargs)

    def test_create_certificate_identifier(self):
        """
        Test that a CertificateIdentifier attribute can be created.
        """
        kwargs = {'name': enums.AttributeType.CERTIFICATE_IDENTIFIER,
                  'value': None}
        self.assertRaises(
            NotImplementedError, self.factory.create_attribute_value, **kwargs)

    def test_create_certificate_subject(self):
        """
        Test that a CertificateSubject attribute can be created.
        """
        kwargs = {'name': enums.AttributeType.CERTIFICATE_SUBJECT,
                  'value': None}
        self.assertRaises(
            NotImplementedError, self.factory.create_attribute_value, **kwargs)

    def test_create_certificate_issuer(self):
        """
        Test that a CertificateIssuer attribute can be created.
        """
        kwargs = {'name': enums.AttributeType.CERTIFICATE_ISSUER,
                  'value': None}
        self.assertRaises(
            NotImplementedError, self.factory.create_attribute_value, **kwargs)

    def test_create_digital_signature_algorithm(self):
        """
        Test that a DigitalSignatureAlgorithm attribute can be created.
        """
        kwargs = {'name': enums.AttributeType.DIGITAL_SIGNATURE_ALGORITHM,
                  'value': None}
        self.assertRaises(
            NotImplementedError, self.factory.create_attribute_value, **kwargs)

    def test_create_digest(self):
        """
        Test that a Digest attribute can be created.
        """
        digest = self.factory.create_attribute_value(
            enums.AttributeType.DIGEST, None)
        self.assertIsInstance(digest, attributes.Digest)

    def test_create_operation_policy_name(self):
        """
        Test that an OperationPolicyName attribute can be created.
        """
        attribute = self.factory.create_attribute_value(
            enums.AttributeType.OPERATION_POLICY_NAME, 'test')
        self.assertIsInstance(attribute, attributes.OperationPolicyName)
        self.assertEqual('test', attribute.value)

    def test_create_cryptographic_usage_mask(self):
        """
        Test that a CryptographicUsageMask attribute can be created.
        """
        self.skipTest('')

    def test_create_lease_time(self):
        """
        Test that a LeaseTime attribute can be created.
        """
        lease = self.factory.create_attribute_value(
            enums.AttributeType.LEASE_TIME, 0)
        self.assertIsInstance(lease, primitives.Interval)
        self.assertEqual(0, lease.value)
        self.assertEqual(enums.Tags.LEASE_TIME, lease.tag)

    def test_create_usage_limits(self):
        """
        Test that a UsageLimits attribute can be created.
        """
        kwargs = {'name': enums.AttributeType.USAGE_LIMITS,
                  'value': None}
        self.assertRaises(
            NotImplementedError, self.factory.create_attribute_value, **kwargs)

    def test_create_state(self):
        """
        Test that a State attribute can be created.
        """
        state = self.factory.create_attribute_value(
            enums.AttributeType.STATE,
            enums.State.ACTIVE
        )
        self.assertIsInstance(state, attributes.State)
        self.assertEqual(enums.State.ACTIVE, state.value)

    def test_create_initial_date(self):
        """
        Test that an InitialDate attribute can be created.
        """
        date = self.factory.create_attribute_value(
            enums.AttributeType.INITIAL_DATE, 0)
        self.assertIsInstance(date, primitives.DateTime)
        self.assertEqual(0, date.value)
        self.assertEqual(enums.Tags.INITIAL_DATE, date.tag)

    def test_create_activation_date(self):
        """
        Test that an ActivationDate attribute can be created.
        """
        date = self.factory.create_attribute_value(
            enums.AttributeType.ACTIVATION_DATE, 0)
        self.assertIsInstance(date, primitives.DateTime)
        self.assertEqual(0, date.value)
        self.assertEqual(enums.Tags.ACTIVATION_DATE, date.tag)

    def test_create_process_start_date(self):
        """
        Test that a ProcessStartDate attribute can be created.
        """
        date = self.factory.create_attribute_value(
            enums.AttributeType.PROCESS_START_DATE, 0)
        self.assertIsInstance(date, primitives.DateTime)
        self.assertEqual(0, date.value)
        self.assertEqual(enums.Tags.PROCESS_START_DATE, date.tag)

    def test_create_protect_stop_date(self):
        """
        Test that a ProtectStopDate attribute can be created.
        """
        date = self.factory.create_attribute_value(
            enums.AttributeType.PROTECT_STOP_DATE, 0)
        self.assertIsInstance(date, primitives.DateTime)
        self.assertEqual(0, date.value)
        self.assertEqual(enums.Tags.PROTECT_STOP_DATE, date.tag)

    def test_create_deactivation_date(self):
        """
        Test that a DeactivationDate attribute can be created.
        """
        date = self.factory.create_attribute_value(
            enums.AttributeType.DEACTIVATION_DATE, 0)
        self.assertIsInstance(date, primitives.DateTime)
        self.assertEqual(0, date.value)
        self.assertEqual(enums.Tags.DEACTIVATION_DATE, date.tag)

    def test_create_destroy_date(self):
        """
        Test that a DestroyDate attribute can be created.
        """
        date = self.factory.create_attribute_value(
            enums.AttributeType.DESTROY_DATE, 0)
        self.assertIsInstance(date, primitives.DateTime)
        self.assertEqual(0, date.value)
        self.assertEqual(enums.Tags.DESTROY_DATE, date.tag)

    def test_create_compromise_occurance_date(self):
        """
        Test that a CompromiseOccurrenceDate attribute can be created.
        """
        date = self.factory.create_attribute_value(
            enums.AttributeType.COMPROMISE_OCCURRENCE_DATE, 0)
        self.assertIsInstance(date, primitives.DateTime)
        self.assertEqual(0, date.value)
        self.assertEqual(enums.Tags.COMPROMISE_OCCURRENCE_DATE, date.tag)

    def test_create_compromise_date(self):
        """
        Test that a CompromiseDate attribute can be created.
        """
        date = self.factory.create_attribute_value(
            enums.AttributeType.COMPROMISE_DATE, 0)
        self.assertIsInstance(date, primitives.DateTime)
        self.assertEqual(0, date.value)
        self.assertEqual(enums.Tags.COMPROMISE_DATE, date.tag)

    def test_create_revocation_reason(self):
        """
        Test that a RevocationReason attribute can be created.
        """
        kwargs = {'name': enums.AttributeType.REVOCATION_REASON,
                  'value': None}
        self.assertRaises(
            NotImplementedError, self.factory.create_attribute_value, **kwargs)

    def test_create_archive_date(self):
        """
        Test that an ArchiveDate attribute can be created.
        """
        date = self.factory.create_attribute_value(
            enums.AttributeType.ARCHIVE_DATE, 0)
        self.assertIsInstance(date, primitives.DateTime)
        self.assertEqual(0, date.value)
        self.assertEqual(enums.Tags.ARCHIVE_DATE, date.tag)

    def test_create_object_group(self):
        """
        Test that an ObjectGroup attribute can be created.
        """
        object_group = self.factory.create_attribute_value(
            enums.AttributeType.OBJECT_GROUP,
            "Group1"
        )
        self.assertIsInstance(object_group, primitives.TextString)
        self.assertEqual("Group1", object_group.value)
        self.assertEqual(enums.Tags.OBJECT_GROUP, object_group.tag)

        object_group = self.factory.create_attribute_value_by_enum(
            enums.Tags.OBJECT_GROUP,
            "Group1"
        )
        self.assertIsInstance(object_group, primitives.TextString)
        self.assertEqual("Group1", object_group.value)
        self.assertEqual(enums.Tags.OBJECT_GROUP, object_group.tag)

    def test_create_fresh(self):
        """
        Test that a Fresh attribute can be created.
        """
        fresh = self.factory.create_attribute_value(
            enums.AttributeType.FRESH, True)
        self.assertIsInstance(fresh, primitives.Boolean)
        self.assertEqual(True, fresh.value)
        self.assertEqual(enums.Tags.FRESH, fresh.tag)

    def test_create_link(self):
        """
        Test that a Link attribute can be created.
        """
        kwargs = {'name': enums.AttributeType.LINK,
                  'value': None}
        self.assertRaises(
            NotImplementedError, self.factory.create_attribute_value, **kwargs)

    def test_create_application_specific_information(self):
        """
        Test that an ApplicationSpecificInformation attribute can be created.
        """
        attribute = self.factory.create_attribute_value(
            enums.AttributeType.APPLICATION_SPECIFIC_INFORMATION,
            {
                "application_namespace": "ssl",
                "application_data": "www.example.com"
            }
        )
        self.assertIsInstance(
            attribute,
            attributes.ApplicationSpecificInformation
        )
        self.assertEqual("ssl", attribute.application_namespace)
        self.assertEqual("www.example.com", attribute.application_data)

        attribute = self.factory.create_attribute_value(
            enums.AttributeType.APPLICATION_SPECIFIC_INFORMATION,
            None
        )
        self.assertIsInstance(
            attribute,
            attributes.ApplicationSpecificInformation
        )
        self.assertIsNone(attribute.application_namespace)
        self.assertIsNone(attribute.application_data)

        attribute = self.factory.create_attribute_value_by_enum(
            enums.Tags.APPLICATION_SPECIFIC_INFORMATION,
            {
                "application_namespace": "ssl",
                "application_data": "www.example.com"
            }
        )
        self.assertIsInstance(
            attribute,
            attributes.ApplicationSpecificInformation
        )
        self.assertEqual("ssl", attribute.application_namespace)
        self.assertEqual("www.example.com", attribute.application_data)

        attribute = self.factory.create_attribute_value_by_enum(
            enums.Tags.APPLICATION_SPECIFIC_INFORMATION,
            None
        )
        self.assertIsInstance(
            attribute,
            attributes.ApplicationSpecificInformation
        )
        self.assertIsNone(attribute.application_namespace)
        self.assertIsNone(attribute.application_data)

    def test_create_contact_information(self):
        """
        Test that a ContactInformation attribute can be created.
        """
        self.skipTest('')

    def test_create_last_change_date(self):
        """
        Test that an LastChangeDate attribute can be created.
        """
        date = self.factory.create_attribute_value(
            enums.AttributeType.LAST_CHANGE_DATE, 0)
        self.assertIsInstance(date, primitives.DateTime)
        self.assertEqual(0, date.value)
        self.assertEqual(enums.Tags.LAST_CHANGE_DATE, date.tag)

    def test_create_custom_attribute(self):
        """
        Test that a CustomAttribute can be created.
        """
        custom = self.factory.create_attribute_value(
            enums.AttributeType.CUSTOM_ATTRIBUTE, None)
        self.assertIsInstance(custom, attributes.CustomAttribute)

    def test_create_sensitive(self):
        """
        Test that a Sensitive attribute can be created.
        """
        sensitive = self.factory.create_attribute_value(
            enums.AttributeType.SENSITIVE,
            True
        )
        self.assertIsInstance(sensitive, primitives.Boolean)
        self.assertTrue(sensitive.value)
        self.assertEqual(enums.Tags.SENSITIVE, sensitive.tag)

        sensitive = self.factory.create_attribute_value_by_enum(
            enums.Tags.SENSITIVE,
            False
        )
        self.assertIsInstance(sensitive, primitives.Boolean)
        self.assertFalse(sensitive.value)
        self.assertEqual(enums.Tags.SENSITIVE, sensitive.tag)
